﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WINDF7CSharp
{
    public partial class FormReport : Form
    {
        public FormReport()
        {
            InitializeComponent();
        }

        private void FormReport_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
            this.Hide();
            toolStripButtonPreview.ToolTipText = "Druckvorschau";
            toolStripButton42StoreTxt.ToolTipText = "Speichere Report als .txt- Datei";
            toolStripButton41Clear.ToolTipText = "Lösche Report";
            toolStripButton32SetUp.ToolTipText = "Druckerseiten einrichten";
            toolStripButton31Print.ToolTipText = "Drucke Report";

        }

        private int startSeite;		    // Anzahl der Druckseiten
        private int anzahlSeiten;		// aktuelle Seitenzahl
        private int seitenNummer;		// zu druckender Text
        private string strPrintText;    // der Name der geöffneten Datei
        string strDateiName = "";

 

        private void toolStripButton31Print_Click(object sender, EventArgs e)
        {
            printDialog1.AllowSomePages = true;
            if (printDialog1.ShowDialog() == DialogResult.OK)
            {
                printDocument1.DocumentName = "Dokument";
                // Startwerte abhängig vom zu druckenden Text initialisieren
                switch (printDialog1.PrinterSettings.PrintRange)
                {
                    case PrintRange.AllPages:
                        strPrintText = tBMainReport.Text;
                        startSeite = 1;
                        anzahlSeiten = printDialog1.PrinterSettings.MaximumPage;
                        break;
                    case PrintRange.SomePages:
                        strPrintText = tBMainReport.Text;
                        startSeite = printDialog1.PrinterSettings.FromPage;
                        anzahlSeiten = printDialog1.PrinterSettings.ToPage - startSeite + 1;
                        break;
                }
                // Drucken starten
                seitenNummer = 1;
                printDocument1.Print();
            }

        }

        private void toolStripButton32SetUp_Click(object sender, EventArgs e)
        {
            pageSetupDialog1.ShowDialog();
        }

        private void toolStripButtonPreview_Click(object sender, EventArgs e)
        {
            printDialog1.AllowSomePages = true;
            strPrintText = tBMainReport.Text;
            startSeite = 1;
            anzahlSeiten = printDialog1.PrinterSettings.MaximumPage;
            seitenNummer = 1;
            printPreviewDialog1.ShowDialog();
        }

        private void toolStripButton41Clear_Click(object sender, EventArgs e)
        {
            tBMainReport.Clear();
        }

  
        private void tsbHistory_Click(object sender, EventArgs e)
        {
            StreamReader filestream = new StreamReader("history.txt", Encoding.Default);
            string line = "", crlf = "\r\n";
            tBMainReport.Visible = false;
            while (line != null)
            {
                line = filestream.ReadLine();
                if (line != null)
                {
                    tBMainReport.AppendText(line + crlf);
                }
            }
            tBMainReport.Visible = true;
            filestream.Close();
        }

        public void english()
        {
            toolStripButtonPreview.ToolTipText = "Printing preview";
            toolStripButton42StoreTxt.ToolTipText = "Store report as .txt- file";
            toolStripButton41Clear.ToolTipText = "Clear Report";
            toolStripButton32SetUp.ToolTipText = "Setup printer";
            toolStripButton31Print.ToolTipText = "Print Report";
        }

        private void toolStripButton42StoreTxt_Click_1(object sender, EventArgs e)
        {
            if (tBMainReport.Lines.Length > 0)
                if (saveFileDialogReport.ShowDialog() == DialogResult.OK)
                {
                    StreamWriter myStream1 = new StreamWriter(saveFileDialogReport.OpenFile(), Encoding.Default);
                    myStream1.WriteLine(tBMainReport.Text);
                    myStream1.Close();
                }


        }

        private void printDocument1_PrintPage(object sender, PrintPageEventArgs e)
        {
            StringFormat stringFormat = new StringFormat();
            RectangleF rectFPapier, rectFText;
            int intChars, intLines;

            // Ermitteln des Rectangles, das den gesamten Druckbereich
            // beschreibt (inklusive Kopf- und Fusszeile)
            rectFPapier = e.MarginBounds;

            // Ermitteln des Rectangles, das den Bereich für den
            // Text beschreibt (ausschließlich Kopf- und Fusszeile)
            rectFText = RectangleF.Inflate(rectFPapier, 0, -2 * tBMainReport.Font.GetHeight(e.Graphics));

            // eine gerade Anzahl von Druckzeilen ermitteln
            int anzahlZeilen = (int)Math.Floor(rectFText.Height / tBMainReport.Font.GetHeight(e.Graphics));

            // die Höhe die textbeinhaltenden Rechtecks festlegen, damit die
            // letzte Druckzeile nicht abgeschnitten wird
            rectFText.Height = anzahlZeilen * tBMainReport.Font.GetHeight(e.Graphics);

            // das StringFormat-Objekt festlegen, um den Text in einem Rechteck
            // anzuzeigen - Text bis zum nächstliegenden Wort verkürzen
            stringFormat.Trimming = StringTrimming.Word;

            //	legt die Druckstartseite fest, wenn es sich nicht um die
            // erste Dokumentenseite handelt
            while ((seitenNummer < startSeite) && (strPrintText.Length > 0))
            {
                e.Graphics.MeasureString(strPrintText, tBMainReport.Font, rectFText.Size, stringFormat, out intChars, out intLines);
                strPrintText = strPrintText.Substring(intChars);
                seitenNummer++;
            }

            // Druckjob beenden, wenn es keinen Text zum Drucken mehr gibt
            if (strPrintText.Length == 0)
            {
                e.Cancel = true;
                return;
            }

            // den Text an das Graphics-Objekt übergeben
            e.Graphics.DrawString(strPrintText, tBMainReport.Font, Brushes.Black, rectFText, stringFormat);

            // Text für die nächste Seite
            // intChars - Anzahl der Zeichen in der Zeichenfolge
            // intLines - Anzahl der Zeilen in der Zeichenfolge
            e.Graphics.MeasureString(strPrintText, tBMainReport.Font, rectFText.Size, stringFormat, out intChars, out intLines);
            strPrintText = strPrintText.Substring(intChars);

            // StringFormat restaurieren
            stringFormat = new StringFormat();

            // Dateiname in der Kopfzeile anzeigen
            stringFormat.Alignment = StringAlignment.Center;
            e.Graphics.DrawString(this.strDateiName, tBMainReport.Font, Brushes.Black, rectFPapier, stringFormat);

            // Seitennummer in der Fusszeile anzeigen
            stringFormat.LineAlignment = StringAlignment.Far;
            e.Graphics.DrawString("Page " + seitenNummer, tBMainReport.Font, Brushes.Black, rectFPapier, stringFormat);

            // ermitteln, ob weitere Seiten zu drucken sind
            seitenNummer++;
            e.HasMorePages = (strPrintText.Length > 0) && (seitenNummer < startSeite + anzahlSeiten);

            // Neuinitialisierung
            if (!e.HasMorePages)
            {
                strPrintText = tBMainReport.Text;
                startSeite = 1;
                anzahlSeiten = printDialog1.PrinterSettings.MaximumPage;
                seitenNummer = 1;
            }


        }

        private void cBTopMost_CheckedChanged(object sender, EventArgs e)
        {
            this.TopMost = cBTopMost.Checked;
        }
    }
}
