﻿/**************************************************************************************
 * Description  : Development of a Tool to Edit and Run Fuzzy Controllers Integrated  *
 *                in the Visual C# Project WindfC#                                    *
 * Author       : Noor Azamiah binti Abdul Rashid                                     *
 **************************************************************************************/




using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;       //provides access to GDI+ basic graphics functionality
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using WINDF7CSharp;

namespace fuzzy
{
    public partial class FormFuzEditor : Form
    {
        Graphics gPB1;          //****************************************************//
        Graphics gPB2;          //                                                    //
        Graphics gPB3;          // provides methods for drawing to the display device //
        Graphics gPB4;          // Encapsulates a GDI+ drawing surface                //
        Graphics gPB5;          //                                                    //
        Graphics gPB6;          //****************************************************//

        FormFuzSetEdit frm2;
        FormMain meinMain;
        public FormFuzEditor(FormMain m)
        {
            InitializeComponent();
            meinMain = m;
            gPB1 = Graphics.FromHwnd(panel1.Handle);
            gPB2 = Graphics.FromHwnd(panel2.Handle);
            gPB3 = Graphics.FromHwnd(panel3.Handle);
            gPB4 = Graphics.FromHwnd(panel4.Handle);
            gPB5 = Graphics.FromHwnd(panel5.Handle);
            gPB6 = Graphics.FromHwnd(panel6.Handle);

            frm2 = new FormFuzSetEdit(this); //FormFuzSetEdit is form2 to edit Fuzzy set

            // create Arrays //
            Seta = new double[inmax, setmax]; // Sets
            Setb = new double[inmax, setmax]; // Sets
            Setc = new double[inmax, setmax]; // Sets
            Setd = new double[inmax, setmax]; // Sets
            setout = new double[setmax];  // output-singletons 
            rule = new int[ixout + 1, rulemax];  // Rule-matrix 
            nbsets = new int[ixout]; //number of Sets per Fuzzy variable
            Typ = new double[inmax, setmax]; 
            // Type of membership function/fuzzy-set:"0" for Trapezoid and "1" for Triangle
           
        }
        public const int rulemax = 100;//define a maximum rule
        public const int inmax = 2;//define maximum inputs
        public const int setmax = 7;//define maximum sets
        const int ixout = inmax + 1;
        public double[,] Typ;
        public double[,] Seta, Setb, Setc, Setd; // Sets
        public double[] setout;  // output-singletons 
        public int[,] rule;  // Rule-matrix 
        public int[] nbsets; //number of Sets per Fuzzy variable
        public int nbrule;   // number of rules
        public int nbin;     //nbin : Number of inputs 
        public string outproc; //outproc: either "hold" or "zero" 
        double holdoutp;    // internal memory

        public int endOfSetsIn2;

        string warnungdopplteRegel1 = "The rule is already exist";
        string warnungdopplteRegel2 = "The rule is already exist";
        string warnungdopplteRegel3 = "The rule is already exist";
        string warnungdopplteRegel4 = "The rule is already exist";
        string weight = "Weight/%";
        
        public void deutsch()
        {
            buttonLoadFuz.Text = "Laden";
            buttonDraw.Text = "Zeichne";
            buttonDrawArray.Text = "Zeichne von Array";
            buttonSaveAs.Text = "Speichen Unter";
            buttonSave.Text = "Speichen";
            buttonCompute.Text = "Berechne";
            button_clear.Text = "Loeschen";
            buttonDeleteRule.Text = "Selektierte Regel Loeschen";
            buttonDelAllRules.Text = "Komplette Regelbasis Loeschen";
            warnungdopplteRegel1 = "Regel existiert schon";
            warnungdopplteRegel2 = "Regel existiert schon";
            warnungdopplteRegel3 = "Regel existiert schon";
            warnungdopplteRegel4 = "Regel existiert schon";
            weight = "Gewichtung/%";
            groupBoxOutFuz.Text = "output wenn keine Regeln schiesst";
            rBOutFuzZero.Text = "Null";
            rBOutFuzHold.Text = "Halten";
        }

        public int Str2NString(string txt, ref string[] val, char sep)
        {// separator defined with RadioButtons in this Form
            // separates the input string txt into any number of substrings separated with separator 
            // output string array is val. Returns the number of strings, which are found
            //char sep = getSeparator();
            string sepstr = sep.ToString();
            int startx, chnb, cnt = 0;
            int pos, px = 0;
            if (txt.Length > 2)
            {// Minimum Lengh is 3 char
                while (txt.Substring(px, 1) == sepstr) px++; //skip leading seps
                startx = px;
                pos = px + 1;
                while ((px >= 0) && (pos < txt.Length))  // search for all separators
                {
                    px = txt.IndexOf(sep, pos);
                    chnb = px - startx;
                    //if more than one sep skip//
                    while ((px > -1) && (px < txt.Length - 1) && (txt.Substring(px + 1, 1) == sepstr)) px++; 

                    if ((px < txt.Length) && (px >= 0))
                    {
                        val[cnt] = txt.Substring(startx, chnb);
                        cnt++;
                        pos = px + 1;
                        startx = pos;
                    }
                }
                if ((startx != txt.Length) && (px < 0)) // No sep at the end
                {
                    val[cnt] = txt.Substring(startx, txt.Length - startx);
                    cnt++;
                }
            }
            else return (-1);
            return (cnt);
        }

        public double fuzzy(double[] fuzin)//  fuzin : input values
        {
            //****** Runtime function Fuzzy control ******//
            // local arrays and variables //
            double[,] mue = new double[inmax, setmax];  // µ- Werte condition
            double[] outmue = new double[setmax];     // µ-Werte result
            double slopeleft, slopert, inp, mu, nom, denom;
            double a, b, c, d, lhs, help1, outp;
            int i, j;

            /* Fuzzification */
            for (i = 0; i < nbin; i++)
            {
                inp = fuzin[i];
                for (j = 0; j < nbsets[i]; j++)
                {  /* compute the actual values of mue for (each set*/
                    /* first compute slopes of set */
                    a = Seta[i, j]; b = Setb[i, j]; c = Setc[i, j]; d = Setd[i, j];
                    if (Math.Abs(b - a) > 1e-10) slopeleft = 1.0 / (b - a); else slopeleft = 1e10;
                    if (Math.Abs(c - d) > 1e-10) slopert = 1.0 / (c - d); else slopert = 1e10;
                    /* compute mue */
                    if ((inp <= a) || (inp >= d)) mu = 0.0;
                    else
                        if ((inp >= b) && (inp <= c)) mu = 1.0;
                        else
                            if ((inp > a) && (inp < b)) mu = (inp - a) * slopeleft;
                            else mu = (inp - d) * slopert;
                    mue[i, j] = mu;
                } /* for (j...*/
            } /*for (i... */

            /* Inference */
            for (i = 0; i < setmax; i++) outmue[i] = 0; /* init of output-mue */
            for (i = 0; i < nbrule; i++)
            {
                /* LeftHandSide-funktion is Minimum */
                lhs = 1.0;
                for (j = 0; j < nbin; j++)
                {
                    help1 = mue[j, rule[j, i] - 1];
                    if (help1 < lhs) lhs = help1;
                }
                lhs = lhs * rule[3, i] / 100.0; // Weight / Gewichtung
                /* RightHandSide-Function is Maximum */
                if (lhs > outmue[rule[ixout - 1, i] - 1]) outmue[rule[ixout - 1, i] - 1] = lhs;
            }   /*for (i...*/ ;

            // Defuzzyfication : cog with singletons //
            if (rBOutFuzZero.Checked)
            {
                outproc = "zero";
            }
            else
            {
                outproc = "hold";
            }
            nom = 0; denom = 0;
            for (i = 0; i < nbsets[ixout - 1]; i++)
            {
                denom = denom + outmue[i] * setout[i];
                nom = nom + outmue[i];
            }
            if (Math.Abs(nom) > 1e-10)
            {
                outp = denom / nom;
                holdoutp = outp;
            }
            else if (outproc == "zero") outp = 0;
            else outp = holdoutp;
            return (outp);
        } /*  end fuzzy runtime */

        public String[,] inOutnames = new String[1, 3];//name of Fuzzy variable
        public String[,] names = new String[2, 14];//name and abbreviation of Input1's Fuzzy set
        public String[,] names2 = new String[2, 14];//name and abbreviation of Input2's Fuzzy set
        public String[,] Outnames = new String[2, 14];//name and abbreviation of Output's Fuzzy set
        
        char separator = ' ';
        String[] test = new String[1000];
        public int rules;
        public int sttRules;
        public string FuzFileName="";
         
        public void buttonLoadFuz_Click(object sender, EventArgs e)  //load fuz.file
        {
            button_clear_Click(sender, null);            
            changeOK = false;
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                StreamReader myStream1 = new StreamReader(openFileDialog1.OpenFile());
                saveFileDialog1.FileName = openFileDialog1.FileName;
                FuzFileName = openFileDialog1.FileName;
                string item = myStream1.ReadToEnd();
                textBox1.Text = item;
                myStream1.Close();
            }
            buttonDraw_Click(sender, null); //draw
        }

        public void buttonDraw_Click(object sender, EventArgs e) 
        {
            /*draw fuzzy-sets of input1, input2 and ouput, and
             *create dataTables called "liste" and "Matrix" to show rulebase by 
             *using datasets extracted from fuz.file which has been loaded previously
             */
            changeOK = false;

            // clear all lists of Fuzzy sets //
            richTextBox1.Text = "";    
            richTextBox2.Text = "";    
            richTextBox3.Text = "";     

            // clear all Fuzzy sets //
            SolidBrush clear = new SolidBrush(Color.White);
            gPB1.FillRectangle(clear, 0, 0, panel1.Width, panel1.Height);       
            gPB2.FillRectangle(clear, 0, 0, panel2.Width, panel2.Height);       
            gPB3.FillRectangle(clear, 0, 0, panel3.Width, panel3.Height);        
            gPB4.FillRectangle(clear, 0, 0, panel4.Width, panel4.Height);      
            gPB5.FillRectangle(clear, 0, 0, panel5.Width, panel5.Height);       
            gPB6.FillRectangle(clear, 0, 0, panel6.Width, panel6.Height);      

            // clear rules list (Liste) //
            dataGridView1.Rows.Clear();
            dataGridView1.Columns.Clear();

            //****** First Input / first linguistic variable ******//
            if (textBox1.Lines.Length > 0)
            {
                label1.Show();
                label2.Show();
                label3.Show();
                label4.Show();
                label5.Show();
                label6.Show();
                label8.Show();
                panel3.Show();
                panel4.Show();
                richTextBox2.Show();
                textBoxIn2.Show();
                
                label4.Text = textBox1.Lines[11];//display linguistic variable of input1 above the input1 frame
                label1.Text = textBox1.Lines[11];//display linguistic variable of input1 in Matrix
                inOutnames[0, 0] = textBox1.Lines[11];//store linguistic variable of input1 in Array

                // draw Frame //
                Pen p = new Pen(Color.Black, 4);
                gPB2.DrawRectangle(p, 0, 0, panel2.Width, panel2.Height);

                // set string font and colour to draw Graph Scale //
                Font drawFont = new Font("Arial", 7);
                SolidBrush drawBrush = new SolidBrush(Color.Black);

                // draw Graph Scale for x-axis // (1)
                string Xaxis = textBox1.Lines[13];
                int nb = Str2NString(Xaxis, ref test, separator);
                double xminn = 0;
                meinMain.ConvertToDouble(test[0], ref xminn);
                int xmin = (int)(xminn);//min value of x-axis
                double xmaxx = 10;
                meinMain.ConvertToDouble(test[1], ref xmaxx);
                int xmax = (int)(xmaxx);//max value of x-axis
                int diff = xmax - xmin;//range of x-axis

                /* numbering the x-scale from min to max value in steps of one */
                PointF drawPoint0 = new PointF(18F, 260F);
                gPB1.DrawString(xmin.ToString(), drawFont, drawBrush, drawPoint0);

                PointF drawPoint1 = new PointF(53F, 260F);
                gPB1.DrawString((xmin + diff / 10).ToString(), drawFont, drawBrush, drawPoint1);

                PointF drawPoint2 = new PointF(89F, 260F);
                gPB1.DrawString((xmin + diff / 5).ToString(), drawFont, drawBrush, drawPoint2);

                PointF drawPoint3 = new PointF(126F, 260F);
                gPB1.DrawString((xmin + diff / 10 * 3).ToString(), drawFont, drawBrush, drawPoint3);

                PointF drawPoint4 = new PointF(162F, 260F);
                gPB1.DrawString((xmin + diff / 10 * 4).ToString(), drawFont, drawBrush, drawPoint4);

                PointF drawPoint5 = new PointF(201F, 260F);
                gPB1.DrawString((xmin + diff / 2).ToString(), drawFont, drawBrush, drawPoint5);

                PointF drawPoint6 = new PointF(238F, 260F);
                gPB1.DrawString((xmin + diff / 10 * 6).ToString(), drawFont, drawBrush, drawPoint6);

                PointF drawPoint7 = new PointF(273F, 260F);
                gPB1.DrawString((xmin + diff / 10 * 7).ToString(), drawFont, drawBrush, drawPoint7);

                PointF drawPoint8 = new PointF(310F, 260F);
                gPB1.DrawString((xmin + diff / 10 * 8).ToString(), drawFont, drawBrush, drawPoint8);

                PointF drawPoint9 = new PointF(346F, 260F);
                gPB1.DrawString((xmin + diff / 10 * 9).ToString(), drawFont, drawBrush, drawPoint9);

                PointF drawPoint10 = new PointF(380F, 260F);
                gPB1.DrawString((xmax).ToString(), drawFont, drawBrush, drawPoint10);

                // draw Graph Scale for y-axis //
                PointF drawPointy0 = new PointF(0F, 250F);
                gPB1.DrawString("0.0", drawFont, drawBrush, drawPointy0);

                PointF drawPointy1 = new PointF(0F, 213F);
                gPB1.DrawString("0.2", drawFont, drawBrush, drawPointy1);

                PointF drawPointy2 = new PointF(0F, 172F);
                gPB1.DrawString("0.4", drawFont, drawBrush, drawPointy2);

                PointF drawPointy3 = new PointF(0F, 131F);
                gPB1.DrawString("0.6", drawFont, drawBrush, drawPointy3);

                PointF drawPointy4 = new PointF(0F, 92F);
                gPB1.DrawString("0.8", drawFont, drawBrush, drawPointy4);

                PointF drawPointy5 = new PointF(0F, 51F);
                gPB1.DrawString("1.0", drawFont, drawBrush, drawPointy5);

                PointF drawPointy6 = new PointF(0F, 10F);
                gPB1.DrawString("1.2", drawFont, drawBrush, drawPointy6);

                // set gray colour to draw Grid lines //
                Pen grayline = new Pen(Color.Gray, 0);
                /* Vertical Grid lines */
                gPB2.DrawLine(grayline, panel2.Width / 10, 0, panel2.Width / 10, panel2.Height);
                gPB2.DrawLine(grayline, panel2.Width * 2 / 10, 0, panel2.Width * 2 / 10, panel2.Height);
                gPB2.DrawLine(grayline, panel2.Width * 3 / 10, 0, panel2.Width * 3 / 10, panel2.Height);
                gPB2.DrawLine(grayline, panel2.Width * 4 / 10, 0, panel2.Width * 4 / 10, panel2.Height);
                gPB2.DrawLine(grayline, panel2.Width * 5 / 10, 0, panel2.Width * 5 / 10, panel2.Height);
                gPB2.DrawLine(grayline, panel2.Width * 6 / 10, 0, panel2.Width * 6 / 10, panel2.Height);
                gPB2.DrawLine(grayline, panel2.Width * 7 / 10, 0, panel2.Width * 7 / 10, panel2.Height);
                gPB2.DrawLine(grayline, panel2.Width * 8 / 10, 0, panel2.Width * 8 / 10, panel2.Height);
                gPB2.DrawLine(grayline, panel2.Width * 9 / 10, 0, panel2.Width * 9 / 10, panel2.Height);
                /* Horizontal Grid lines */
                gPB2.DrawLine(grayline, 0, panel2.Height / 6, panel2.Width, panel2.Height / 6);
                gPB2.DrawLine(grayline, 0, panel2.Height * 2 / 6, panel2.Width, panel2.Height * 2 / 6);
                gPB2.DrawLine(grayline, 0, panel2.Height * 3 / 6, panel2.Width, panel2.Height * 3 / 6);
                gPB2.DrawLine(grayline, 0, panel2.Height * 4 / 6, panel2.Width, panel2.Height * 4 / 6);
                gPB2.DrawLine(grayline, 0, panel2.Height * 5 / 6, panel2.Width, panel2.Height * 5 / 6);

                // store total of fuzzy sets for input1 in Array // (2)
                double setss = 7;
                meinMain.ConvertToDouble(textBox1.Lines[14], ref setss);
                int sets = (int)(setss);
                nbsets[0] = sets;

                // Rules in Matrix //
                dataGridView2.RowCount = sets;//number of rows depending on nbsets[0]
                dataGridView2.RowHeadersWidth = 90;

                //**** draw fuzzy sets of 1st linguistic variable in input1 frame (panel2) ****//
                //starting from 15th line of Fuzzy file which shows the 1st fuzzy set's name
                int startline = 15;
                int endOfSets;//last line used to draw input1 fuzzy sets
                //only line 15th till line "endofsets", used to draw input1 fuzzy sets
                endOfSets = startline + sets * 4;
                for (int st = 18; st < endOfSets; st++)
                {
                    string set1 = textBox1.Lines[st];
                    int nbb = Str2NString(set1, ref test, separator);

                    double inp1Pt00a = 0;
                    meinMain.ConvertToDouble(test[0], ref inp1Pt00a);//Set a 
                    double inp1Pt11d = 0;
                    meinMain.ConvertToDouble(test[1], ref inp1Pt11d);//Set d 
                    double inp1Pt33b = 0;
                    meinMain.ConvertToDouble(test[3], ref inp1Pt33b);//Set b 
                    double inp1Pt44c = 0;
                    meinMain.ConvertToDouble(test[4], ref inp1Pt44c);//Set c 

                    int inp1Pt0a = (int)(((inp1Pt00a - xmin) / (xmax - xmin)) * panel2.Width);
                    int inp1Pt1d = (int)(((inp1Pt11d - xmin) / (xmax - xmin)) * panel2.Width);
                    int inp1Pt3b = (int)(((inp1Pt33b - xmin) / (xmax - xmin)) * panel2.Width);
                    int inp1Pt4c = (int)(((inp1Pt44c - xmin) / (xmax - xmin)) * panel2.Width);

                    /* 1st input1 fuzzy set */
                    if (st == 18)
                    {
                        Pen botright = new Pen(Color.Magenta, 2);//set line colour to draw fuzzy set
                        //draw Fuzzy set in trapezoidal form if Typ = 0//
                        if (textBox1.Lines[17] == "0")
                        {
                            //store data in arrays//
                            Typ[0, 0] = 0;//trapezoidal form
                            Seta[0, 0] = inp1Pt00a;//starting point of set
                            Setb[0, 0] = inp1Pt33b;//inflexion point 1
                            Setc[0, 0] = inp1Pt44c;//inflexion point 2
                            Setd[0, 0] = inp1Pt11d;//endpoint

                            //draw fuzzy set//
                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt4c, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt4c, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }
                        //draw Fussy set in triangular form if Typ = 1//
                        if (textBox1.Lines[17] == "1")
                        {
                            //store data in arrays//
                            Typ[0, 0] = 1;//triangular form
                            Seta[0, 0] = inp1Pt00a;
                            Setb[0, 0] = inp1Pt33b;
                            Setc[0, 0] = inp1Pt33b;
                            Setd[0, 0] = inp1Pt11d;

                            //draw fuzzy set//
                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }
                        
                        //write name of fuzzy-set in the list next to input1 frame//
                        //string colour should be the same as the colour of drawn fuzzy set
                        richTextBox1.SelectedText = "";
                        richTextBox1.SelectionColor = Color.Magenta;
                        richTextBox1.SelectedText = textBox1.Lines[15];

                        //store name and its abbreviation of fuzzy-set in array//
                        names[0, 0] = textBox1.Lines[15];//name of fuzzy set; e.g: botright
                        names[1, 0] = textBox1.Lines[16];//abbreviation of fuzzy set; e.g: br
                    }

                    /* 2nd input1 fuzzy set */
                    //the programming method below is similar to the 1st input1 fuzzy set Section//
                    if (st == 22)
                    {
                        Pen botright = new Pen(Color.Red, 2);
                        if (textBox1.Lines[21] == "0")
                        {
                            Typ[0, 1] = 0;
                            Seta[0, 1] = inp1Pt00a;
                            Setb[0, 1] = inp1Pt33b;
                            Setc[0, 1] = inp1Pt44c;
                            Setd[0, 1] = inp1Pt11d;
                            
                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt4c, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt4c, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }
                        if (textBox1.Lines[21] == "1")
                        {
                            Typ[0, 1] = 1;
                            Seta[0, 1] = inp1Pt00a;
                            Setb[0, 1] = inp1Pt33b;
                            Setc[0, 1] = inp1Pt33b;
                            Setd[0, 1] = inp1Pt11d;
                            
                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }                        

                        richTextBox1.SelectedText = "\r\n";
                        richTextBox1.SelectionColor = Color.Red;
                        richTextBox1.SelectedText = textBox1.Lines[19];

                        names[0, 1] = textBox1.Lines[19];
                        names[1, 1] = textBox1.Lines[20];
                    }
                    /* 3rd input1 fuzzy set */
                    //the programming method below is similar to the 1st input1 fuzzy set Section//
                    if (st == 26)
                    {
                        Pen botright = new Pen(Color.Green, 2);
                        if (textBox1.Lines[25] == "0")
                        {
                            Typ[0, 2] = 0;
                            Seta[0, 2] = inp1Pt00a;
                            Setb[0, 2] = inp1Pt33b;
                            Setc[0, 2] = inp1Pt44c;
                            Setd[0, 2] = inp1Pt11d;

                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt4c, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt4c, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }
                        if (textBox1.Lines[25] == "1")
                        {
                            Typ[0, 2] = 1;
                            Seta[0, 2] = inp1Pt00a;
                            Setb[0, 2] = inp1Pt33b;
                            Setc[0, 2] = inp1Pt33b;
                            Setd[0, 2] = inp1Pt11d;

                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }
                        
                        richTextBox1.SelectedText = "\r\n";
                        richTextBox1.SelectionColor = Color.Green;
                        richTextBox1.SelectedText = textBox1.Lines[23];

                        names[0, 2] = textBox1.Lines[23];
                        names[1, 2] = textBox1.Lines[24];
                    }
                    /* 4th input1 fuzzy set */
                    //the programming method below is similar to the 1st input1 fuzzy set Section//
                    if (st == 30)
                    {
                        Pen botright = new Pen(Color.Blue, 2);
                        if (textBox1.Lines[29] == "0")
                        {
                            Typ[0, 3] = 0;
                            Seta[0, 3] = inp1Pt00a;
                            Setb[0, 3] = inp1Pt33b;
                            Setc[0, 3] = inp1Pt44c;
                            Setd[0, 3] = inp1Pt11d;

                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt4c, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt4c, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }
                        if (textBox1.Lines[29] == "1")
                        {
                            Typ[0, 3] = 1;
                            Seta[0, 3] = inp1Pt00a;
                            Setb[0, 3] = inp1Pt33b;
                            Setc[0, 3] = inp1Pt33b;
                            Setd[0, 3] = inp1Pt11d;

                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }

                        richTextBox1.SelectedText = "\r\n";
                        richTextBox1.SelectionColor = Color.Blue;
                        richTextBox1.SelectedText = textBox1.Lines[27];

                        names[0, 3] = textBox1.Lines[27];
                        names[1, 3] = textBox1.Lines[28];
                    }
                    /* 5th input1 fuzzy set */
                    //the programming method below is similar to the 1st input1 fuzzy set Section//
                    if (st == 34)
                    {
                        Pen botright = new Pen(Color.Orange, 2);
                        if (textBox1.Lines[33] == "0")
                        {
                            Typ[0, 4] = 0;
                            Seta[0, 4] = inp1Pt00a;
                            Setb[0, 4] = inp1Pt33b;
                            Setc[0, 4] = inp1Pt44c;
                            Setd[0, 4] = inp1Pt11d;

                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt4c, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt4c, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }
                        if (textBox1.Lines[33] == "1")
                        {
                            Typ[0, 4] = 1;
                            Seta[0, 4] = inp1Pt00a;
                            Setb[0, 4] = inp1Pt33b;
                            Setc[0, 4] = inp1Pt33b;
                            Setd[0, 4] = inp1Pt11d;

                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }

                        richTextBox1.SelectedText = "\r\n";
                        richTextBox1.SelectionColor = Color.Orange;
                        richTextBox1.SelectedText = textBox1.Lines[31];

                        names[0, 4] = textBox1.Lines[31];
                        names[1, 4] = textBox1.Lines[32];
                    }
                    /* 6th input1 fuzzy set */
                    //the programming method below is similar to the 1st input1 fuzzy set Section//
                    if (st == 38)
                    {
                        Pen botright = new Pen(Color.Black, 2);                        
                        if (textBox1.Lines[37] == "0")
                        {
                            Typ[0, 5] = 0;
                            Seta[0, 5] = inp1Pt00a;
                            Setb[0, 5] = inp1Pt33b;
                            Setc[0, 5] = inp1Pt44c;
                            Setd[0, 5] = inp1Pt11d;

                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt4c, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt4c, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }
                        if (textBox1.Lines[37] == "1")
                        {
                            Typ[0, 5] = 1;
                            Seta[0, 5] = inp1Pt00a;
                            Setb[0, 5] = inp1Pt33b;
                            Setc[0, 5] = inp1Pt33b;
                            Setd[0, 5] = inp1Pt11d;

                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }

                        richTextBox1.SelectedText = "\r\n";
                        richTextBox1.SelectionColor = Color.Black;
                        richTextBox1.SelectedText = textBox1.Lines[35];

                        names[0, 5] = textBox1.Lines[35];
                        names[1, 5] = textBox1.Lines[36];
                    }
                    /* 7th input1 fuzzy set */
                    //the programming method below is similar to the 1st input1 fuzzy set Section//
                    if (st == 42)
                    {
                        Pen botright = new Pen(Color.Cyan, 2);
                        if (textBox1.Lines[41] == "0")
                        {
                            Typ[0, 6] = 0;
                            Seta[0, 6] = inp1Pt00a;
                            Setb[0, 6] = inp1Pt33b;
                            Setc[0, 6] = inp1Pt44c;
                            Setd[0, 6] = inp1Pt11d;

                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt4c, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt4c, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }
                        if (textBox1.Lines[41] == "1")
                        {
                            Typ[0, 6] = 1;
                            Seta[0, 6] = inp1Pt00a;
                            Setb[0, 6] = inp1Pt33b;
                            Setc[0, 6] = inp1Pt33b;
                            Setd[0, 6] = inp1Pt11d;

                            gPB2.DrawLine(botright, inp1Pt0a, panel2.Height, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt3b, panel2.Height / 6);
                            gPB2.DrawLine(botright, inp1Pt3b, panel2.Height / 6, inp1Pt1d, panel2.Height);
                        }

                        richTextBox1.SelectedText = "\r\n";
                        richTextBox1.SelectionColor = Color.Cyan;
                        richTextBox1.SelectedText = textBox1.Lines[39];

                        names[0, 6] = textBox1.Lines[39];
                        names[1, 6] = textBox1.Lines[40];
                    }

                    st = st + 3;
                }

                //****** Second Input / Second linguistic variable ******// (7)
                double InOutputt = 3;
                meinMain.ConvertToDouble(textBox1.Lines[10], ref InOutputt);
                int InOutput = (int)(InOutputt);//input + output
                int input;
                input = InOutput - 1;//number of inputs =InOutput - 1 output
                nbin = input;//nbin / number of inputs
                if (input == 2)//draw only when number of inputs is 2
                {
                    label5.Text = textBox1.Lines[endOfSets];//display linguistic variable of input2 above the input1 frame
                    label2.Text = textBox1.Lines[endOfSets];//display linguistic variable of input2 in Matrix
                    inOutnames[0, 1] = textBox1.Lines[endOfSets];//store linguistic variable of input2 in Array
                    
                    // draw Frame//
                    gPB4.DrawRectangle(p, 0, 0, panel4.Width, panel4.Height);

                    // draw Graph Scale for x-axis // 
                    string Xaxis2 = textBox1.Lines[endOfSets + 2];
                    int nbbb = Str2NString(Xaxis2, ref test, separator);
                    double xminIn2 = 0;
                    meinMain.ConvertToDouble(test[0], ref xminIn2);//min value of x-axis (8)
                    double xmaxIn2 = 0;
                    meinMain.ConvertToDouble(test[1], ref xmaxIn2);//max value of x-axis (9)
                    double diff2 = xmaxIn2 - xminIn2;//range of x-axis

                    /* numbering the x-scale from min to max value in steps of one */
                    //all drawpoints are similar to input1's drawpoints//
                    gPB3.DrawString(xminIn2.ToString(), drawFont, drawBrush, drawPoint0);
                    gPB3.DrawString((xminIn2 + diff2 / 10).ToString(), drawFont, drawBrush, drawPoint1);
                    gPB3.DrawString((xminIn2 + diff2 / 5).ToString(), drawFont, drawBrush, drawPoint2);
                    gPB3.DrawString((xminIn2 + diff2 / 10 * 3).ToString(), drawFont, drawBrush, drawPoint3);
                    gPB3.DrawString((xminIn2 + diff2 / 10 * 4).ToString(), drawFont, drawBrush, drawPoint4);
                    gPB3.DrawString((xminIn2 + diff2 / 2).ToString(), drawFont, drawBrush, drawPoint5);
                    gPB3.DrawString((xminIn2 + diff2 / 10 * 6).ToString(), drawFont, drawBrush, drawPoint6);
                    gPB3.DrawString((xminIn2 + diff2 / 10 * 7).ToString(), drawFont, drawBrush, drawPoint7);
                    gPB3.DrawString((xminIn2 + diff2 / 10 * 8).ToString(), drawFont, drawBrush, drawPoint8);
                    gPB3.DrawString((xminIn2 + diff2 / 10 * 9).ToString(), drawFont, drawBrush, drawPoint9);
                    gPB3.DrawString((xmaxIn2).ToString(), drawFont, drawBrush, drawPoint10);

                    // draw Graph Scale for y-axis //
                    //all drawpoints were set previously in input1's section//
                    gPB3.DrawString("0.0", drawFont, drawBrush, drawPointy0);
                    gPB3.DrawString("0.2", drawFont, drawBrush, drawPointy1);
                    gPB3.DrawString("0.4", drawFont, drawBrush, drawPointy2);
                    gPB3.DrawString("0.6", drawFont, drawBrush, drawPointy3);
                    gPB3.DrawString("0.8", drawFont, drawBrush, drawPointy4);
                    gPB3.DrawString("1.0", drawFont, drawBrush, drawPointy5);
                    gPB3.DrawString("1.2", drawFont, drawBrush, drawPointy6);

                    //Gray softlines Darstellung

                    /* Vertical Grid lines */
                    //colour of grayline was set previously in input1's section//
                    gPB4.DrawLine(grayline, panel4.Width / 10, 0, panel4.Width / 10, panel4.Height);
                    gPB4.DrawLine(grayline, panel4.Width * 2 / 10, 0, panel4.Width * 2 / 10, panel4.Height);
                    gPB4.DrawLine(grayline, panel4.Width * 3 / 10, 0, panel4.Width * 3 / 10, panel4.Height);
                    gPB4.DrawLine(grayline, panel4.Width * 4 / 10, 0, panel4.Width * 4 / 10, panel4.Height);
                    gPB4.DrawLine(grayline, panel4.Width * 5 / 10, 0, panel4.Width * 5 / 10, panel4.Height);
                    gPB4.DrawLine(grayline, panel4.Width * 6 / 10, 0, panel4.Width * 6 / 10, panel4.Height);
                    gPB4.DrawLine(grayline, panel4.Width * 7 / 10, 0, panel4.Width * 7 / 10, panel4.Height);
                    gPB4.DrawLine(grayline, panel4.Width * 8 / 10, 0, panel4.Width * 8 / 10, panel4.Height);
                    gPB4.DrawLine(grayline, panel4.Width * 9 / 10, 0, panel4.Width * 9 / 10, panel4.Height);
                    /* Horizontal Grid lines */
                    gPB4.DrawLine(grayline, 0, panel4.Height / 6, panel4.Width, panel4.Height / 6);
                    gPB4.DrawLine(grayline, 0, panel4.Height * 2 / 6, panel4.Width, panel4.Height * 2 / 6);
                    gPB4.DrawLine(grayline, 0, panel4.Height * 3 / 6, panel4.Width, panel4.Height * 3 / 6);
                    gPB4.DrawLine(grayline, 0, panel4.Height * 4 / 6, panel4.Width, panel4.Height * 4 / 6);
                    gPB4.DrawLine(grayline, 0, panel4.Height * 5 / 6, panel4.Width, panel4.Height * 5 / 6);

                    // store total of fuzzy sets for input2 in Array //
                    double setsIn22 = 0;
                    meinMain.ConvertToDouble(textBox1.Lines[endOfSets + 3],ref setsIn22);
                    int setsIn2 = (int)(setsIn22);
                    nbsets[1] = setsIn2;
                    // Rules in Matrix //
                    dataGridView2.ColumnCount = setsIn2;//set how many columns depends on nbsets[2]

                    //**** draw fuzzy sets of 2nd linguistic variable in input2 frame (panel4) ****//
                    //public int endOfSetsIn2
                    endOfSetsIn2 = endOfSets + 4 + setsIn2 * 4;//last line which is used to draw input2 fuzzy sets
                    //start line used to draw input2 fuzzy sets which shows the 1st fuzzy set's name
                    int stt2 = endOfSets + 7;
                    for (int st2 = endOfSets + 7; st2 < endOfSetsIn2; st2++)
                    {
                        string set2 = textBox1.Lines[st2];
                        int nbbbb = Str2NString(set2, ref test, separator);

                        double inp2Pt00a = 0;
                        meinMain.ConvertToDouble(test[0], ref inp2Pt00a);//Set a  
                        double inp2Pt11d = 0;
                        meinMain.ConvertToDouble(test[1], ref inp2Pt11d);//Set d  
                        double inp2Pt33b = 0;
                        meinMain.ConvertToDouble(test[3], ref inp2Pt33b);//Set b  
                        double inp2Pt44c = 0;
                        meinMain.ConvertToDouble(test[4], ref inp2Pt44c);//Set c  

                        int inp2Pt0a = (int)((inp2Pt00a - xminIn2) / (xmaxIn2 - xminIn2) * panel4.Width);
                        int inp2Pt1d = (int)((inp2Pt11d - xminIn2) / (xmaxIn2 - xminIn2) * panel4.Width);
                        int inp2Pt3b = (int)((inp2Pt33b - xminIn2) / (xmaxIn2 - xminIn2) * panel4.Width);
                        int inp2Pt4c = (int)((inp2Pt44c - xminIn2) / (xmaxIn2 - xminIn2) * panel4.Width);

                        /* 1st input2 fuzzy set */
                        if (st2 == stt2)
                        {
                            Pen botright = new Pen(Color.Magenta, 2);//set line colour to draw fuzzy set
                            //Fuzzy set in trapezoidal form//
                            if (textBox1.Lines[stt2 - 1] == "0")
                            {
                                //store data in arrays//
                                Typ[1, 0] = 0;//trapezoidal form
                                Seta[1, 0] = inp2Pt00a;//starting point of set
                                Setb[1, 0] = inp2Pt33b;//inflexion point 1
                                Setc[1, 0] = inp2Pt44c;//inflexion point 2
                                Setd[1, 0] = inp2Pt11d;//endpoint

                                //draw fuzzy set//
                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt4c, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt4c, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }
                            //Fuzzy set in triangular form//
                            if (textBox1.Lines[stt2 - 1] == "1")
                            {
                                Typ[1, 0] = 1;
                                Seta[1, 0] = inp2Pt00a;
                                Setb[1, 0] = inp2Pt33b;
                                Setc[1, 0] = inp2Pt33b;
                                Setd[1, 0] = inp2Pt11d;

                                //draw fuzzy set//
                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }

                            //write name of fuzzy-set in the list next to input2 frame//
                            //string colour should be the same as the colour of drawn fuzzy set
                            richTextBox2.SelectedText = "";
                            richTextBox2.SelectionColor = Color.Magenta;
                            richTextBox2.SelectedText = textBox1.Lines[stt2 - 3];

                            //store name and its abbreviation of fuzzy-set in array//
                            names2[0, 0] = textBox1.Lines[stt2 - 3];
                            names2[1, 0] = textBox1.Lines[stt2 - 3 + 1];
                        }

                        /* 2nd input1 fuzzy set */
                        //the programming method below is similar to the 1st input2 fuzzy set Section//
                        if (st2 == stt2 + 4)
                        {
                            Pen botright = new Pen(Color.Red, 2);
                            if (textBox1.Lines[stt2 + 4 - 1] == "0")
                            {
                                Typ[1, 1] = 0;
                                Seta[1, 1] = inp2Pt00a;
                                Setb[1, 1] = inp2Pt33b;
                                Setc[1, 1] = inp2Pt44c;
                                Setd[1, 1] = inp2Pt11d;

                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt4c, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt4c, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }
                            if (textBox1.Lines[stt2 + 4 - 1] == "1")
                            {
                                Typ[1, 1] = 1;
                                Seta[1, 1] = inp2Pt00a;
                                Setb[1, 1] = inp2Pt33b;
                                Setc[1, 1] = inp2Pt33b;
                                Setd[1, 1] = inp2Pt11d;

                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }

                            richTextBox2.SelectedText = "\r\n";
                            richTextBox2.SelectionColor = Color.Red;
                            richTextBox2.SelectedText = textBox1.Lines[stt2 + 4 - 3];

                            names2[0, 1] = textBox1.Lines[stt2 + 4 - 3];
                            names2[1, 1] = textBox1.Lines[stt2 + 4 - 3 + 1];
                        }
                        /* 3rd input1 fuzzy set */
                        //the programming method below is similar to the 1st input2 fuzzy set Section//
                        if (st2 == stt2 + 8)
                        {
                            Pen botright = new Pen(Color.Green, 2);
                            if (textBox1.Lines[stt2 + 8 - 1] == "0")
                            {
                                Typ[1, 2] = 0;
                                Seta[1, 2] = inp2Pt00a;
                                Setb[1, 2] = inp2Pt33b;
                                Setc[1, 2] = inp2Pt44c;
                                Setd[1, 2] = inp2Pt11d;

                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt4c, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt4c, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }
                            if (textBox1.Lines[stt2 + 8 - 1] == "1")
                            {
                                Typ[1, 2] = 1;
                                Seta[1, 2] = inp2Pt00a;
                                Setb[1, 2] = inp2Pt33b;
                                Setc[1, 2] = inp2Pt33b;
                                Setd[1, 2] = inp2Pt11d;

                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }

                            richTextBox2.SelectedText = "\r\n";
                            richTextBox2.SelectionColor = Color.Green;
                            richTextBox2.SelectedText = textBox1.Lines[stt2 + 8 - 3];

                            names2[0, 2] = textBox1.Lines[stt2 + 8 - 3];
                            names2[1, 2] = textBox1.Lines[stt2 + 8 - 3 + 1];
                        }
                        /* 4th input1 fuzzy set */
                        //the programming method below is similar to the 1st input2 fuzzy set Section//
                        if (st2 == stt2 + 12)
                        {
                            Pen botright = new Pen(Color.Blue, 2);
                            if (textBox1.Lines[stt2 + 12 - 1] == "0")
                            {
                                Typ[1, 3] = 0;
                                Seta[1, 3] = inp2Pt00a;
                                Setb[1, 3] = inp2Pt33b;
                                Setc[1, 3] = inp2Pt44c;
                                Setd[1, 3] = inp2Pt11d;

                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt4c, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt4c, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }
                            if (textBox1.Lines[stt2 + 12 - 1] == "1")
                            {
                                Typ[1, 3] = 1;
                                Seta[1, 3] = inp2Pt00a;
                                Setb[1, 3] = inp2Pt33b;
                                Setc[1, 3] = inp2Pt33b;
                                Setd[1, 3] = inp2Pt11d;

                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }

                            richTextBox2.SelectedText = "\r\n";
                            richTextBox2.SelectionColor = Color.Blue;
                            richTextBox2.SelectedText = textBox1.Lines[stt2 + 12 - 3];

                            names2[0, 3] = textBox1.Lines[stt2 + 12 - 3];
                            names2[1, 3] = textBox1.Lines[stt2 + 12 - 3 + 1];
                        }
                        /* 5th input1 fuzzy set */
                        //the programming method below is similar to the 1st input2 fuzzy set Section//
                        if (st2 == stt2 + 16)
                        {
                            Pen botright = new Pen(Color.Orange, 2);
                            if (textBox1.Lines[stt2 + 16 - 1] == "0")
                            {
                                Typ[1, 4] = 0;
                                Seta[1, 4] = inp2Pt00a;
                                Setb[1, 4] = inp2Pt33b;
                                Setc[1, 4] = inp2Pt44c;
                                Setd[1, 4] = inp2Pt11d;

                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt4c, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt4c, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }
                            if (textBox1.Lines[stt2 + 16 - 1] == "1")
                            {
                                Typ[1, 4] = 1;
                                Seta[1, 4] = inp2Pt00a;
                                Setb[1, 4] = inp2Pt33b;
                                Setc[1, 4] = inp2Pt33b;
                                Setd[1, 4] = inp2Pt11d;

                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }

                            richTextBox2.SelectedText = "\r\n";
                            richTextBox2.SelectionColor = Color.Orange;
                            richTextBox2.SelectedText = textBox1.Lines[stt2 + 16 - 3];

                            names2[0, 4] = textBox1.Lines[stt2 + 16 - 3];
                            names2[1, 4] = textBox1.Lines[stt2 + 16 - 3 + 1];
                        }
                        /* 6th input1 fuzzy set */
                        //the programming method below is similar to the 1st input2 fuzzy set Section//
                        if (st2 == stt2 + 20)
                        {
                            Pen botright = new Pen(Color.Black, 2);
                            if (textBox1.Lines[stt2 + 20 - 1] == "0")
                            {
                                Typ[1, 5] = 0;
                                Seta[1, 5] = inp2Pt00a;
                                Setb[1, 5] = inp2Pt33b;
                                Setc[1, 5] = inp2Pt44c;
                                Setd[1, 5] = inp2Pt11d;

                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt4c, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt4c, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }
                            if (textBox1.Lines[stt2 + 20 - 1] == "1")
                            {
                                Typ[1, 5] = 1;
                                Seta[1, 5] = inp2Pt00a;
                                Setb[1, 5] = inp2Pt33b;
                                Setc[1, 5] = inp2Pt33b;
                                Setd[1, 5] = inp2Pt11d;

                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }

                            richTextBox2.SelectedText = "\r\n";
                            richTextBox2.SelectionColor = Color.Black;
                            richTextBox2.SelectedText = textBox1.Lines[stt2 + 20 - 3];

                            names2[0, 5] = textBox1.Lines[stt2 + 20 - 3];
                            names2[1, 5] = textBox1.Lines[stt2 + 20 - 3 + 1];
                        }
                        /* 7th input1 fuzzy set */
                        //the programming method below is similar to the 1st input2 fuzzy set Section//
                        if (st2 == stt2 + 24)
                        {
                            Pen botright = new Pen(Color.Cyan, 2);
                            if (textBox1.Lines[stt2 + 24 - 1] == "0")
                            {
                                Typ[1, 6] = 0;
                                Seta[1, 6] = inp2Pt00a;
                                Setb[1, 6] = inp2Pt33b;
                                Setc[1, 6] = inp2Pt44c;
                                Setd[1, 6] = inp2Pt11d;

                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt4c, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt4c, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }
                            if (textBox1.Lines[stt2 + 24 - 1] == "1")
                            {
                                Typ[1, 6] = 1;
                                Seta[1, 6] = inp2Pt00a;
                                Setb[1, 6] = inp2Pt33b;
                                Setc[1, 6] = inp2Pt33b;
                                Setd[1, 6] = inp2Pt11d;

                                gPB4.DrawLine(botright, inp2Pt0a, panel4.Height, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt3b, panel4.Height / 6);
                                gPB4.DrawLine(botright, inp2Pt3b, panel4.Height / 6, inp2Pt1d, panel4.Height);
                            }

                            richTextBox2.SelectedText = "\r\n";
                            richTextBox2.SelectionColor = Color.Cyan;
                            richTextBox2.SelectedText = textBox1.Lines[stt2 + 24 - 3];
                            names2[0, 6] = textBox1.Lines[stt2 + 24 - 3];
                            names2[1, 6] = textBox1.Lines[stt2 + 24 - 3 + 1];
                        }

                        st2 = st2 + 3;
                    }
                }
                if (nbin == 1)
                {
                    dataGridView2.Visible = false;
                    panel3.Visible = false;
                    panel4.Visible = false;
                    label5.Visible = false;
                    label1.Visible = false;
                    label2.Visible = false;
                    label8.Visible = false;
                    textBoxIn2.Visible = false;
                    label9.Visible = false;
                    richTextBox2.Visible = false;
                }

                //****** Output / 3rd linguistic variable ******//     
                    int endline;
                    if(nbin==2)//if number of inputs is 2
                    {
                        endline=endOfSetsIn2;
                    }
                    else//if number of inputs is not 2
                    {
                        endline=endOfSets;
                    }
                    //display linguistic variable of output above the output frame//
                    //store linguistic variable of output in Array//
                    label6.Text = textBox1.Lines[endline];
                    inOutnames[0, 2] = textBox1.Lines[endline];

                    // draw Frame //
                    gPB6.DrawRectangle(p, 0, 0, panel6.Width, panel6.Height);

                    // draw Graph Scale for x-axis // 
                    string XaxisOutput = textBox1.Lines[endline + 2];
                    int nbbbbb = Str2NString(XaxisOutput, ref test, separator);
                    double xminOutt = 0;
                    meinMain.ConvertToDouble(test[0], ref xminOutt);
                    int xminOut = (int)(xminOutt);//min value of x-axis 
                    double xmaxOutt = 0;
                    meinMain.ConvertToDouble(test[1], ref xmaxOutt);
                    int xmaxOut = (int)(xmaxOutt);//max value of x-axis 
                    int diffOut = xmaxOut - xminOut;//range of x-axis

                    /* numbering the x-scale from min to max value in steps of one */
                    gPB5.DrawString(xminOut.ToString(), drawFont, drawBrush, drawPoint0);
                    gPB5.DrawString((xminOut + diffOut / 10).ToString(), drawFont, drawBrush, drawPoint1);
                    gPB5.DrawString((xminOut + diffOut / 5).ToString(), drawFont, drawBrush, drawPoint2);
                    gPB5.DrawString((xminOut + diffOut / 10 * 3).ToString(), drawFont, drawBrush, drawPoint3);
                    gPB5.DrawString((xminOut + diffOut / 10 * 4).ToString(), drawFont, drawBrush, drawPoint4);
                    gPB5.DrawString((xminOut + diffOut / 2).ToString(), drawFont, drawBrush, drawPoint5);
                    gPB5.DrawString((xminOut + diffOut / 10 * 6).ToString(), drawFont, drawBrush, drawPoint6);
                    gPB5.DrawString((xminOut + diffOut / 10 * 7).ToString(), drawFont, drawBrush, drawPoint7);
                    gPB5.DrawString((xminOut + diffOut / 10 * 8).ToString(), drawFont, drawBrush, drawPoint8);
                    gPB5.DrawString((xminOut + diffOut / 10 * 9).ToString(), drawFont, drawBrush, drawPoint9);
                    gPB5.DrawString((xmaxOut).ToString(), drawFont, drawBrush, drawPoint10);

                    // draw Graph Scale for y-axis //         
                    gPB5.DrawString("0.0", drawFont, drawBrush, drawPointy0);
                    gPB5.DrawString("0.2", drawFont, drawBrush, drawPointy1);
                    gPB5.DrawString("0.4", drawFont, drawBrush, drawPointy2);
                    gPB5.DrawString("0.6", drawFont, drawBrush, drawPointy3);
                    gPB5.DrawString("0.8", drawFont, drawBrush, drawPointy4);
                    gPB5.DrawString("1.0", drawFont, drawBrush, drawPointy5);
                    gPB5.DrawString("1.2", drawFont, drawBrush, drawPointy6);
                    /* Vertical Grid lines */
                    gPB6.DrawLine(grayline, panel6.Width / 10, 0, panel6.Width / 10, panel6.Height);
                    gPB6.DrawLine(grayline, panel6.Width * 2 / 10, 0, panel6.Width * 2 / 10, panel6.Height);
                    gPB6.DrawLine(grayline, panel6.Width * 3 / 10, 0, panel6.Width * 3 / 10, panel6.Height);
                    gPB6.DrawLine(grayline, panel6.Width * 4 / 10, 0, panel6.Width * 4 / 10, panel6.Height);
                    gPB6.DrawLine(grayline, panel6.Width * 5 / 10, 0, panel6.Width * 5 / 10, panel6.Height);
                    gPB6.DrawLine(grayline, panel6.Width * 6 / 10, 0, panel6.Width * 6 / 10, panel6.Height);
                    gPB6.DrawLine(grayline, panel6.Width * 7 / 10, 0, panel6.Width * 7 / 10, panel6.Height);
                    gPB6.DrawLine(grayline, panel6.Width * 8 / 10, 0, panel6.Width * 8 / 10, panel6.Height);
                    gPB6.DrawLine(grayline, panel6.Width * 9 / 10, 0, panel6.Width * 9 / 10, panel6.Height);
                    /* Horizontal Grid lines */
                    gPB6.DrawLine(grayline, 0, panel6.Height / 6, panel6.Width, panel6.Height / 6);
                    gPB6.DrawLine(grayline, 0, panel6.Height * 2 / 6, panel6.Width, panel6.Height * 2 / 6);
                    gPB6.DrawLine(grayline, 0, panel6.Height * 3 / 6, panel6.Width, panel6.Height * 3 / 6);
                    gPB6.DrawLine(grayline, 0, panel6.Height * 4 / 6, panel6.Width, panel6.Height * 4 / 6);
                    gPB6.DrawLine(grayline, 0, panel6.Height * 5 / 6, panel6.Width, panel6.Height * 5 / 6);

                    // store total of fuzzy sets for output in Array // 
                    double setsOutt = 7;
                    meinMain.ConvertToDouble(textBox1.Lines[endline + 3], ref setsOutt);
                    int setsOut = (int)(setsOutt);
                    nbsets[2] = setsOut;
                
                    //****** draw fuzzy sets of 3rd linguistic variable in output frame (panel6) ******//
                    int endOfSetsOut;
                    endOfSetsOut = endline + 4 + setsOut * 4;//last line used to draw output fuzzy sets
                    int sttOut = endline + 7;//start line used to draw fuzzy-set which shows the 1st fuzzy set's name
                    for (int stOut = endline + 7; stOut < endOfSetsOut; stOut++)
                    {
                        string outputvalues = textBox1.Lines[stOut];
                        int nbbbbbb = Str2NString(outputvalues, ref test, separator);
                        double Pto11 = 0;
                        meinMain.ConvertToDouble(test[0], ref Pto11);  //  (18)
                        double Pto1n = (Pto11 - xminOut) * 0.5 * panel6.Width / 10;
                        int Pto1 = (int)Pto1n;//setout

                        /* 1st output fuzzy set */
                        if (stOut == sttOut)
                        {
                            setout[0] = Pto11;//store data in array
                            //draw fuzzy set in singleton
                            Pen botright = new Pen(Color.Magenta, 2);
                            gPB6.DrawLine(botright, Pto1, panel6.Height, Pto1, panel6.Height / 6);

                            //write name of fuzzy-set in the list next to output frame//
                            //string colour should be the same as the colour of drawn fuzzy set
                            richTextBox3.SelectedText = "";
                            richTextBox3.SelectionColor = Color.Magenta;
                            richTextBox3.SelectedText = textBox1.Lines[sttOut - 3];

                            //store name and its abbreviation of fuzzy-set in array//
                            Outnames[0, 0] = textBox1.Lines[sttOut - 3];//name of fuzzy set; e.g: Zero
                            Outnames[1, 0] = textBox1.Lines[sttOut - 3 + 1];//abbreviation of fuzzy set; e.g: ze
                        }

                        /* 2nd output fuzzy set */
                        //the programming method below is similar to the 1st output fuzzy set Section//
                        if (stOut == sttOut + 4)
                        {
                            setout[1] = Pto11;
                            
                            Pen botright = new Pen(Color.Red, 2);
                            gPB6.DrawLine(botright, Pto1, panel6.Height, Pto1, panel6.Height / 6);

                            richTextBox3.SelectedText = "\r\n";
                            richTextBox3.SelectionColor = Color.Red;
                            richTextBox3.SelectedText = textBox1.Lines[sttOut + 4 - 3];
                            Outnames[0, 1] = textBox1.Lines[sttOut + 4 - 3];
                            Outnames[1, 1] = textBox1.Lines[sttOut + 4 - 3 + 1];
                        }
                        /* 3rd output fuzzy set */
                        //the programming method below is similar to the 1st output fuzzy set Section//
                        if (stOut == sttOut + 8)
                        {
                            setout[2] = Pto11;

                            Pen botright = new Pen(Color.Green, 2);
                            gPB6.DrawLine(botright, Pto1, panel6.Height, Pto1, panel6.Height / 6);

                            richTextBox3.SelectedText = "\r\n";
                            richTextBox3.SelectionColor = Color.Green;
                            richTextBox3.SelectedText = textBox1.Lines[sttOut + 8 - 3];
                            Outnames[0, 2] = textBox1.Lines[sttOut + 8 - 3];
                            Outnames[1, 2] = textBox1.Lines[sttOut + 8 - 3 + 1];
                        }
                        /* 4th output fuzzy set */
                        //the programming method below is similar to the 1st output fuzzy set Section//
                        if (stOut == sttOut + 12)
                        {
                            setout[3] = Pto11;

                            Pen botright = new Pen(Color.Blue, 2);
                            gPB6.DrawLine(botright, Pto1, panel6.Height, Pto1, panel6.Height / 6);

                            richTextBox3.SelectedText = "\r\n";
                            richTextBox3.SelectionColor = Color.Blue;
                            richTextBox3.SelectedText = textBox1.Lines[sttOut + 12 - 3];
                            Outnames[0, 3] = textBox1.Lines[sttOut + 12 - 3];
                            Outnames[1, 3] = textBox1.Lines[sttOut + 12 - 3 + 1];
                        }
                        /* 5th output fuzzy set */
                        //the programming method below is similar to the 1st output fuzzy set Section//
                        if (stOut == sttOut + 16)
                        {
                            setout[4] = Pto11;

                            Pen botright = new Pen(Color.Orange, 2);
                            gPB6.DrawLine(botright, Pto1, panel6.Height, Pto1, panel6.Height / 6);

                            richTextBox3.SelectedText = "\r\n";
                            richTextBox3.SelectionColor = Color.Orange;
                            richTextBox3.SelectedText = textBox1.Lines[sttOut + 16 - 3];
                            Outnames[0, 4] = textBox1.Lines[sttOut + 16 - 3];
                            Outnames[1, 4] = textBox1.Lines[sttOut + 16 - 3 + 1];
                        }
                        /* 6th output fuzzy set */
                        //the programming method below is similar to the 1st output fuzzy set Section//
                        if (stOut == sttOut + 20)
                        {
                            setout[5] = Pto11;

                            Pen botright = new Pen(Color.Black, 2);
                            gPB6.DrawLine(botright, Pto1, panel6.Height, Pto1, panel6.Height / 6);

                            richTextBox3.SelectedText = "\r\n";
                            richTextBox3.SelectionColor = Color.Black;
                            richTextBox3.SelectedText = textBox1.Lines[sttOut + 20 - 3];
                            Outnames[0, 5] = textBox1.Lines[sttOut + 20 - 3];
                            Outnames[1, 5] = textBox1.Lines[sttOut + 20 - 3 + 1];
                        }
                        /* 7th output fuzzy set */
                        //the programming method below is similar to the 1st output fuzzy set Section//
                        if (stOut == sttOut + 24)
                        {
                            setout[6] = Pto11;

                            Pen botright = new Pen(Color.Cyan, 2);
                            gPB6.DrawLine(botright, Pto1, panel6.Height, Pto1, panel6.Height / 6);

                            richTextBox3.SelectedText = "\r\n";
                            richTextBox3.SelectionColor = Color.Cyan;
                            richTextBox3.SelectedText = textBox1.Lines[sttOut + 24 - 3];
                            Outnames[0, 6] = textBox1.Lines[sttOut + 24 - 3];
                            Outnames[1, 6] = textBox1.Lines[sttOut + 24 - 3 + 1];
                        }

                        stOut = stOut + 3;
                    }
                //*****************************************end draw fuzzy-sets*****************************************//
                if(nbin==2)
                {
                    //****** Rule base of Fuzzy Control ******//
                    //create a table using dataGridView1 for Rule base in List//
                    /* Liste */
                    dataGridView1.ColumnCount = 4;//set number of columns
                    dataGridView1.ColumnHeadersHeight = 40;
                    dataGridView1.ColumnHeadersVisible = true;//display columnHeader
                    dataGridView1.Columns[0].HeaderText = label4.Text;//display name of input1
                    dataGridView1.Columns[1].HeaderText = label5.Text;//display name of input2
                    dataGridView1.Columns[2].HeaderText = label6.Text;//display name of output
                    dataGridView1.Columns[3].HeaderText = weight;//display "Weight/%"
                    //create a table using dataGridView2 for Rule base in Matrix//
                    /*Matrix*/
                    dataGridView2.Visible = true;
                    dataGridView2.ColumnHeadersHeight = 40;
                        for (int k = 0; k < dataGridView2.ColumnCount; k++)
                        {
                            //write all names of input2 in columnsheader cells
                            dataGridView2.Columns[k].HeaderText = richTextBox2.Lines[k];
                        }
                        for (int q = 0; q < dataGridView2.RowCount; q++)
                        {
                            //write all names of input1 in rowsheader cells
                            dataGridView2.Rows[q].HeaderCell.Value = richTextBox1.Lines[q];
                        }
                    //********end create tables*********//
                    // display number of rules //
                    double ruless = 0;
                    meinMain.ConvertToDouble(textBox1.Lines[endOfSetsOut], ref ruless);
                    rules = (int)(ruless);
                    nbrule = rules;
                    label3.Text = nbrule.ToString() + " Rules";                        
                    // write all rules in "Liste" table and "Matrix" table //    
                    sttRules = endOfSetsOut + 1;
                        for (int stRules = 0; stRules < rules; stRules++)
                        {
                            //extract important data and convert to int to be used to write rules in 
                            //tables from fuz.file that has been loaded previously 
                            string ruleValues = textBox1.Lines[stRules + sttRules];
                            int rb = Str2NString(ruleValues, ref test, separator);

                            double input11 = 0;
                            meinMain.ConvertToDouble(test[0], ref input11);  
                            int input1 = (int)(input11);

                            double input22 = 0;
                            meinMain.ConvertToDouble(test[1], ref input22);  
                            int input2 = (int)(input22);

                            double outputt = 0;
                            meinMain.ConvertToDouble(test[2], ref outputt);  
                            int output = (int)(outputt);

                            double gewichtung = 0;
                            meinMain.ConvertToDouble(test[3],ref gewichtung);  //  (23)
                            // store rules in array //
                            rule[0, stRules] = input1;
                            rule[1, stRules] = input2;
                            rule[2, stRules] = output;
                            rule[3, stRules] = (int)(gewichtung * 100);

                            string setname1 = richTextBox1.Lines[input1 - 1];
                            string setname2 = richTextBox2.Lines[input2 - 1];
                            string outputname = richTextBox3.Lines[output - 1];

                            //write rules in tables
                            /*Liste*/
                            dataGridView1.Rows.Add(setname1, setname2, outputname, rule[3, stRules].ToString());
                            /*Matrix*/
                            dataGridView2.Rows[input1 - 1].Cells[input2 - 1].Value = richTextBox3.Lines[output - 1];

                        }// end of writing rules
                    // write number in each row of "Liste" //
                    dataGridView1.RowCount = (nbrule + 1);
                    dataGridView1.RowHeadersWidth = 100;
                        for (int i = 0; i < dataGridView1.RowCount; i++)
                        {
                            dataGridView1.Rows[i].HeaderCell.Value = (i + 1).ToString();
                        }
                    //create List to store items before paste to listbox1
                    List<string> itemss = new List<string>();
                        for (int a = 0; a < richTextBox1.Lines.Length; a++)
                            itemss.Add(richTextBox1.Lines[a]);
                            listBox1.DataSource = itemss;
                }// end of (nbin==2)

                if (nbin == 1)
                {
                    //****** Rule base of Fuzzy Control ******//
                    //create a table using dataGridView1 for Rule base in List//
                    /* Liste */
                    dataGridView1.ColumnCount = 3;//set the number of columns
                    dataGridView1.ColumnHeadersHeight = 40;
                    dataGridView1.ColumnHeadersVisible = true;//display columnHeader
                    dataGridView1.Columns[0].HeaderText = label4.Text;//display name of input1                    
                    dataGridView1.Columns[1].HeaderText = label6.Text;//display name of output
                    dataGridView1.Columns[2].HeaderText = weight;//display "Weight/%"
                    //create a table using dataGridView2 for Rule base in Matrix//
                    /*Matrix*/
                    dataGridView2.Visible = false;
                    // display number of rules //
                    double ruless = 0;
                    meinMain.ConvertToDouble(textBox1.Lines[endOfSetsOut], ref ruless);  // 
                    rules = (int)(ruless);  
                    nbrule = rules;
                    label3.Text = nbrule.ToString() + " Rules"; 

                    // write all rules in "Liste" table and "Matrix" table //
                    sttRules = endOfSetsOut + 1;
                    for (int stRules = 0; stRules < rules; stRules++)
                    {
                        string ruleValues = textBox1.Lines[stRules + sttRules];
                        int rb = Str2NString(ruleValues, ref test, separator);

                        double input11 = 0;
                        meinMain.ConvertToDouble(test[0], ref input11);  
                        int input1 = (int)(input11);

                        double outputt = 0;
                        meinMain.ConvertToDouble(test[1], ref outputt);  
                        int output = (int)(outputt);

                        double gewichtung = 0;
                        meinMain.ConvertToDouble(test[2], ref gewichtung);  
                        // store rules in array //
                        rule[0, stRules] = input1;
                        rule[2, stRules] = output;
                        rule[3, stRules] = (int)(gewichtung * 100);

                        string setname1 = richTextBox1.Lines[input1 - 1];
                        string outputname = richTextBox3.Lines[output - 1];

                        //write rules in "Liste" table
                        dataGridView1.Rows.Add(setname1, outputname, rule[3, stRules].ToString());
                    }
                    // write number in each row of "Liste" table //
                    dataGridView1.RowCount = (nbrule + 1);
                    dataGridView1.RowHeadersWidth = 100;
                    for (int i = 0; i < dataGridView1.RowCount; i++)
                    {
                        dataGridView1.Rows[i].HeaderCell.Value = (i + 1).ToString();
                    }
                    //create List to store items before paste to listbox1
                    List<string> itemss = new List<string>();
                    for (int a = 0; a < richTextBox1.Lines.Length; a++)
                        itemss.Add(richTextBox1.Lines[a]);
                        listBox1.DataSource = itemss;
                }
            }            
        }
        
        private void panel2_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            //******double click in input1's frame to edit fuzzy set value******//
            frm2.radioButton1.Visible = true;//show radiobutton "Triangle"
            frm2.radioButton2.Visible = true;//show radiobutton "Trapezoid"
            frm2.textBox2.ReadOnly = false;
            frm2.textBox3.ReadOnly = false;
            frm2.textBox7.ReadOnly = false;
            frm2.label11.Text = "Input1";
            if (textBox1.Text != "")
            {
                List<string> items = new List<string>();
                for (int i = 0; i < nbsets[0]; i++)
                    items.Add(names[0, i].ToString());
                List<string> items2 = new List<string>();
                for (int i = 0; i < nbsets[0]; i++)
                    items2.Add(names[1, i].ToString());
                frm2.Show();
                frm2.listBox1.DataSource = items;
                frm2.listBox2.DataSource = items2;
                frm2.listBox1.SelectedIndex = frm2.listBox2.SelectedIndex;
                // extract current values from arrays and paste in corresponding textboxes //
                frm2.textBox5.Text = inOutnames[0, 0];
                frm2.textBox4.Text = names[0, frm2.listBox1.SelectedIndex];
                frm2.textBox6.Text = names[1, frm2.listBox1.SelectedIndex];
                frm2.textBox1.Text = Seta[0, frm2.listBox1.SelectedIndex].ToString();
                frm2.textBox2.Text = Setd[0, frm2.listBox1.SelectedIndex].ToString();
                frm2.textBox3.Text = Setb[0, frm2.listBox1.SelectedIndex].ToString();
                frm2.textBox7.Text = Setc[0, frm2.listBox1.SelectedIndex].ToString();
                if (Typ[0, 0] == 1)//check radiobutton 'Triangle'
                {
                    frm2.radioButton1.Checked = true;
                    frm2.textBox7.ReadOnly = true;                    
                }
                if (Typ[0, 0] == 0)//check radiobutton 'Trapezoid'
                {
                    frm2.radioButton2.Checked = true;
                    frm2.textBox7.ReadOnly = false;                    
                }
            }
        }
        
        private void panel6_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            //******double click in output's frame to edit fuzzy set value******//
            frm2.radioButton1.Visible = false;//output only in singleton, no 'Triangle'
            frm2.radioButton2.Visible = false;//or 'Trapezoid'
            frm2.textBox2.ReadOnly = true;
            frm2.textBox3.ReadOnly = true;
            frm2.textBox7.ReadOnly = true;
            frm2.textBox2.Text = "";
            frm2.textBox3.Text = "";
            frm2.textBox7.Text = "";
            frm2.label11.Text = "Output";
            if (textBox1.Text != "")
            {
                // create Lists to paste output's names and abb in listbox1 and listbox2 //
                List<string> items = new List<string>();
                for (int i = 0; i < nbsets[2]; i++)
                    items.Add(Outnames[0, i].ToString());
                List<string> items2 = new List<string>();
                for (int i = 0; i < nbsets[2]; i++)
                    items2.Add(Outnames[1, i].ToString());
                frm2.Show();
                frm2.listBox1.DataSource = items;
                frm2.listBox2.DataSource = items2;
                frm2.listBox1.SelectedIndex = frm2.listBox2.SelectedIndex;
                // extract current values from arrays and paste in corresponding textboxes //
                frm2.textBox5.Text = inOutnames[0, 2];
                frm2.textBox4.Text = Outnames[0, frm2.listBox1.SelectedIndex];
                frm2.textBox6.Text = Outnames[1, frm2.listBox1.SelectedIndex];
                frm2.textBox1.Text = setout[frm2.listBox1.SelectedIndex].ToString();
            }
        }
       
        private void panel4_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            //******double click in input2's frame to edit fuzzy set value******//
            //refer "private void panel2_MouseDoubleClick" for comments
            frm2.radioButton1.Visible = true;
            frm2.radioButton2.Visible = true;
            frm2.textBox2.ReadOnly = false;
            frm2.textBox3.ReadOnly = false;
            frm2.textBox7.ReadOnly = false;
            frm2.label11.Text = "Input2";
            if (textBox1.Text != "")
            {
                List<string> items = new List<string>();
                for (int i = 0; i < nbsets[1]; i++)
                    items.Add(names2[0, i].ToString());
                List<string> items2 = new List<string>();
                for (int i = 0; i < nbsets[1]; i++)
                    items2.Add(names2[1, i].ToString());
                frm2.Show();
                frm2.listBox1.DataSource = items;
                frm2.listBox2.DataSource = items2;
                frm2.listBox1.SelectedIndex = frm2.listBox2.SelectedIndex;
                frm2.textBox5.Text = inOutnames[0, 1];
                frm2.textBox4.Text = names2[0, frm2.listBox1.SelectedIndex];
                frm2.textBox6.Text = names2[1, frm2.listBox1.SelectedIndex];
                frm2.textBox1.Text = Seta[1, frm2.listBox1.SelectedIndex].ToString();
                frm2.textBox2.Text = Setd[1, frm2.listBox1.SelectedIndex].ToString();
                frm2.textBox3.Text = Setb[1, frm2.listBox1.SelectedIndex].ToString();
                frm2.textBox7.Text = Setc[1, frm2.listBox1.SelectedIndex].ToString();
                if (Typ[1, 0] == 1)
                {
                    frm2.radioButton1.Checked = true;
                    frm2.textBox7.ReadOnly = true;                   
                }
                if (Typ[1, 0] == 0)
                {
                    frm2.radioButton2.Checked = true;
                    frm2.textBox7.ReadOnly = false;                    
                }
            }
        } 
       
        private void panel1_Paint(object sender, PaintEventArgs e)
        {
            buttonDraw_Click(sender, null);//draw the sets
        }

        int c;
        int r;
        private void dataGridView1_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            /*      change listbox1's content next to "Liste" table depending on which 
                    column has been clicked                                        
            */
            changeOK = true;
            c = 0;
            
            List<string> listItems = new List<string>();
            c = e.ColumnIndex;//get column index of the current selectedCell
            r = e.RowIndex;//get row index of the current selectedCell

            if (nbin == 2)//number of input is 2
            {
                if (c == 0)
                {
                    //cell in 1st column is clicked
                    for (int a = 0; a < richTextBox1.Lines.Length; a++)
                        listItems.Add(richTextBox1.Lines[a]);
                        listBox1.DataSource = listItems;//listbox1 shows list of input1's names
                }
                if (c == 1)
                {
                    //cell in 2nd column is clicked
                    for (int a = 0; a < richTextBox2.Lines.Length; a++)
                        listItems.Add(richTextBox2.Lines[a]);
                    listBox1.DataSource = listItems;//listbox1 shows list of input2's names
                }
                if (c == 2)
                {
                    //cell in 3rd column is clicked
                    for (int a = 0; a < richTextBox3.Lines.Length; a++)
                        listItems.Add(richTextBox3.Lines[a]);
                    listBox1.DataSource = listItems;//listbox1 shows list of output's names
                }
                if (c == 3)
                {
                    //cell in 4th column is clicked
                    for (int a = 0; a < listBox1.Text.Length; a++)
                    {
                        listItems = null;
                        listBox1.DataSource = listItems;//listbox1 shows empty list
                    }
                }
            }

            if (nbin == 1)//number of input is 1
            {
                if (c == 0)
                {
                    //cell in 1st column is clicked
                    for (int a = 0; a < richTextBox1.Lines.Length; a++)
                        listItems.Add(richTextBox1.Lines[a]);
                    listBox1.DataSource = listItems;//listbox1 shows list of input1's names
                }
                if (c == 1)
                {
                    //cell in 2nd column is clicked
                    for (int a = 0; a < richTextBox3.Lines.Length; a++)
                        listItems.Add(richTextBox3.Lines[a]);
                    listBox1.DataSource = listItems;//listbox1 shows list of output's names
                }
                if (c == 2)
                {
                    //cell in 3rd column is clicked
                    for (int a = 0; a < listBox1.Text.Length; a++)
                    {
                        listItems = null;
                        listBox1.DataSource = listItems;//listbox1 shows empty list
                    }
                }
            }
        }

        int cc;
        int rr;
        private void dataGridView2_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            //***listbox1 shows list of output's names when any cell in "Matrix" table is clicked***//
            cc = e.ColumnIndex;//get column index of current selected cell
            rr = e.RowIndex;//get row index of current selected cell
            
            List<string> listItems = new List<string>();
            listItems = null;
            listBox1.DataSource = listItems;//clear listbox1
            List<string> listItems2 = new List<string>();
            for (int a = 0; a < richTextBox3.Lines.Length; a++)
                listItems2.Add(richTextBox3.Lines[a]);
                listBox1.DataSource = listItems2;//paste output's names
        }

        private void buttonCompute_Click(object sender, EventArgs e) //compute
        {
            //******calculate fuzzy******//
            double[] inp = new double[nbin];
            inp[0] = Convert.ToDouble(textBoxIn1.Text);
            if (nbin == 2)
            {
                inp[1] = Convert.ToDouble(textBoxIn2.Text);
            }
            // calculate outp using fuzzy runtime function "public double fuzzy(double[] fuzin)" //
            double outp = fuzzy(inp);
            labFuzOut.Text = "Fuz Out = " + outp.ToString();
        }
       
        public void buttonSave_Click(object sender, EventArgs e) //save
        {
            //******save the changes that has been made with the existing file's name******// 
            /*build new file using datasets from arrays in textbox5*/
            textBox5.Text = "[Version]\r\n6.1.5.29\r\n!\r\n!\r\n!\r\n!\r\n!\r\n!\r\n!\r\n!\r\n";
            textBox5.AppendText((nbin+1).ToString()+ "\r\n");//print out number of (input+output)
            textBox5.AppendText(inOutnames[0, 0].ToString() + "\r\n");//print out input1's name
            textBox5.AppendText("E\r\n 0.000E+0000  1.000E+0001  0.000E+0000\r\n");
            textBox5.AppendText(nbsets[0].ToString() + "\r\n");//print out number of input1's fuzzysets 
            for (int i = 0; i < nbsets[0]; i++)
            {
                //print out name, abbreviation, typ and all fuzzyset values of every input1's fuzzyset//
                textBox5.AppendText(names[0, i].ToString() + "\r\n");
                textBox5.AppendText(names[1, i].ToString() + "\r\n");
                textBox5.AppendText(Typ[0,i].ToString() + "\r\n");
                textBox5.AppendText(Seta[0, i].ToString() + "  " + Setd[0, i].ToString() + "  1  " + 
                    Setb[0, i].ToString() + "  " + Setc[0, i].ToString() + "\r\n");
            }//end for
            if (nbin == 2)
            {
                textBox5.AppendText(inOutnames[0, 1].ToString() + "\r\n");//print out input2's name
                textBox5.AppendText("E\r\n-1.000E+0000  1.000E+0000  0.000E+0000\r\n");
                textBox5.AppendText(nbsets[1].ToString() + "\r\n");//print out number of input2's fuzzysets
                for (int i = 0; i < nbsets[1]; i++)
                {
                    //print out name, abbreviation, typ and all fuzzyset values of every input2's fuzzyset//
                    textBox5.AppendText(names2[0, i].ToString() + "\r\n");
                    textBox5.AppendText(names2[1, i].ToString() + "\r\n");
                    textBox5.AppendText(Typ[1, i].ToString() + "\r\n");
                    textBox5.AppendText(Seta[1, i].ToString() + "  " + Setd[1, i].ToString() + "  1  " + 
                        Setb[1, i].ToString() + "  " + Setc[1, i].ToString() + "\r\n");
                }//end for
            }//end if

            textBox5.AppendText(inOutnames[0, 2].ToString() + "\r\n");//print out output's name
            textBox5.AppendText("A\r\n-1.000E+0001  1.000E+0001  0.000E+0000\r\n");
            textBox5.AppendText(nbsets[2].ToString() + "\r\n");//print out number of output's fuzzysets
            for (int i = 0; i < nbsets[2]; i++)
            {
                //print out name, abbreviation and all fuzzyset values of every output's fuzzyset//
                textBox5.AppendText(Outnames[0, i].ToString() + "\r\n");
                textBox5.AppendText(Outnames[1, i].ToString() + "\r\n");
                textBox5.AppendText("2\r\n");
                textBox5.AppendText(setout[i].ToString() + "  10  1  0  0" + "\r\n");
            }//end for
            textBox5.AppendText(nbrule.ToString() + "\r\n");
            for (int i = 0; i < nbrule; i++)
            {
                //print out all values of every rule//
                if (nbin == 2)
                {
                    textBox5.AppendText("  " + rule[0, i].ToString() + "   " + rule[1, i].ToString() 
                        + "   " + rule[2, i].ToString() + "  " + (rule[3, i] / 100.0).ToString() + "\r\n");
                }
                if (nbin == 1)
                {
                    textBox5.AppendText("  " + rule[0, i].ToString() + "   " + rule[2, i].ToString() 
                        + "  " + (rule[3, i] / 100.0).ToString() + "\r\n");
                }
            }//end for
            for (int i = sttRules + rules; i < textBox1.Lines.Length - 1; i++)
            {
                textBox5.AppendText(textBox1.Lines[i]);
                if (i != textBox1.Lines.Length - 2)
                {
                    textBox5.AppendText("\r\n");
                }
            }//end for
            //save text built in textbox5 to the existing fuz.file//
            FileStream fs = new FileStream(saveFileDialog1.FileName, FileMode.Create);
            StreamWriter myStream2 = new
            StreamWriter(fs);
            myStream2.WriteLine(textBox5.Text);
            myStream2.Close();
        }

        private void listBox1_DoubleClick(object sender, EventArgs e)
        {
            /*      paste fuzzyset's name in selected cell according to which item 
                    has been doubleClicked in listbox1
             */
            int ruleexists = 0;//set rule is already exist = 0
            int rowcnt = 0;//set rowcount = 0
            if (tabControl1.SelectedIndex == 0)//********if selected Tab is "Liste"*********//
            {
                if (nbin == 2)    /***if number of input is 2 ***/
                {
                    // look which column current selectedCell is in //
                    if (c == 0)//if selectedCell is in 1st column 
                    {
                        if (dataGridView1.Rows[r].Cells[c].Value == null)//if selectedCell in "Liste" table is empty
                        {
                            if (dataGridView1.Rows[r].Cells[1].Value == null)//and the cell next to it also empty
                            {
                                dataGridView1.Rows[r].Cells[c].Value = listBox1.Items[listBox1.SelectedIndex];
                                // paste the item that has been clicked from listbox1 in selectedCell //
                            }
                            else //if selectedCell is empty but next to it is not
                            {
                                ruleexists = 0;
                                rowcnt = dataGridView1.RowCount;//count the total rows of "Liste" table
                                for (int p = 0; p < rowcnt; p++)
                                {
                                    // check whether the new desired rule to be added is already exist by comparing with each rule//
                                    if (dataGridView1.Rows[p].Cells[1].Value != null)
                                        if (dataGridView1.Rows[p].Cells[1].Value.ToString() == dataGridView1.Rows[r].Cells[1].Value.ToString())
                                        {
                                            if (dataGridView1.Rows[p].Cells[0].Value != null)
                                                if (dataGridView1.Rows[p].Cells[0].Value.ToString() == listBox1.Items[listBox1.SelectedIndex].ToString())
                                                {
                                                    // show message "the rule is already exist" //
                                                    MessageBox.Show(warnungdopplteRegel1);
                                                    ruleexists++;
                                                }//end if
                                        }//end if
                                }//end for
                                if (ruleexists == 0) dataGridView1.Rows[r].Cells[c].Value = listBox1.Items[listBox1.SelectedIndex];
                                //if ruleexists = 0 paste item in selectedCell
                            }
                        }
                        else //if selectedCell in "Liste" table is not empty and in 1st column
                        {
                            if (dataGridView1.Rows[r].Cells[1].Value != null)//if cell next to selectedCell is not empty
                            {
                                ruleexists = 0;
                                rowcnt = dataGridView1.RowCount;
                                for (int p = 0; p < rowcnt; p++)
                                {
                                    // check whether the new desired rule to be added is already exist by comparing with each rule//
                                    if (dataGridView1.Rows[p].Cells[1].Value != null)
                                        if (dataGridView1.Rows[p].Cells[1].Value.ToString() == dataGridView1.Rows[r].Cells[1].Value.ToString())
                                        {
                                            if (dataGridView1.Rows[p].Cells[0].Value != null)
                                                if (dataGridView1.Rows[p].Cells[0].Value.ToString() == listBox1.Items[listBox1.SelectedIndex].ToString())
                                                {
                                                    // show message "the rule is already exist" //
                                                    MessageBox.Show(warnungdopplteRegel2);
                                                    ruleexists++;
                                                }//end if
                                        }//end if
                                }//end for
                                if (ruleexists == 0) dataGridView1.Rows[r].Cells[c].Value = listBox1.Items[listBox1.SelectedIndex];
                                //if ruleexists = 0 paste item in selectedCell
                            }//end if
                        }//end else
                    }//end if selectedCell in 1st colum (c==0)
                    if (c == 1)//if selectedCell is in 2nd column
                    {
                        if (dataGridView1.Rows[r].Cells[c].Value == null)//if selectedCell in "Liste" table is empty
                        {
                            if (dataGridView1.Rows[r].Cells[0].Value == null)//and the cell(1st column) next to it also empty
                            {
                                dataGridView1.Rows[r].Cells[c].Value = listBox1.Items[listBox1.SelectedIndex];
                                //paste item in selectedCell
                            }
                            else
                            {
                                ruleexists = 0;
                                rowcnt = dataGridView1.RowCount;
                                for (int p = 0; p < rowcnt; p++)
                                {
                                    // check whether the new desired rule to be added is already exist by comparing with each rule//
                                    if (dataGridView1.Rows[p].Cells[1].Value != null)
                                        if (dataGridView1.Rows[p].Cells[1].Value.ToString() == listBox1.Items[listBox1.SelectedIndex].ToString())
                                        {
                                            if (dataGridView1.Rows[p].Cells[0].Value != null)
                                                if (dataGridView1.Rows[p].Cells[0].Value.ToString() == dataGridView1.Rows[r].Cells[0].Value.ToString())
                                                {
                                                    // show message "the rule is already exist" //
                                                    MessageBox.Show(warnungdopplteRegel3);
                                                    ruleexists++;
                                                }
                                        }
                                }
                                if (ruleexists == 0) dataGridView1.Rows[r].Cells[c].Value = listBox1.Items[listBox1.SelectedIndex];
                                //if ruleexists = 0 paste item in selectedCell
                            }
                        }//end if selectedCell in "Liste" table is empty
                        else//if selectedCell in "Liste" table is not empty(2nd column)
                        {
                            if (dataGridView1.Rows[r].Cells[0].Value == null)//if cell(in 1st column) next to it is empty
                            {
                                dataGridView1.Rows[r].Cells[c].Value = listBox1.Items[listBox1.SelectedIndex];
                                //paste item in selectedCell
                            }
                            else//if cell(in 1st column) next to it is not empty
                            {
                                ruleexists = 0;
                                rowcnt = dataGridView1.RowCount;
                                for (int p = 0; p < rowcnt; p++)
                                {
                                    // check whether the new desired rule to be added is already exist by comparing with each rule//
                                    if (dataGridView1.Rows[p].Cells[1].Value != null)
                                        if (dataGridView1.Rows[p].Cells[1].Value.ToString() == listBox1.Items[listBox1.SelectedIndex].ToString())
                                        {
                                            if (dataGridView1.Rows[p].Cells[0].Value != null)
                                                if (dataGridView1.Rows[p].Cells[0].Value.ToString() == dataGridView1.Rows[r].Cells[0].Value.ToString())
                                                {
                                                    // show message "the rule is already exist" //
                                                    MessageBox.Show(warnungdopplteRegel4);
                                                    ruleexists++;
                                                }
                                        }
                                }//end for
                                if (ruleexists == 0) dataGridView1.Rows[r].Cells[c].Value = listBox1.Items[listBox1.SelectedIndex];
                                //if ruleexists = 0 paste item in selectedCell
                            }//end else
                        }//end else 
                    }//end if selectedCell in 2nd column (c==1)
                    if (c == 2)//if selectedCell in 3rd column
                    {
                        dataGridView1.Rows[r].Cells[c].Value = listBox1.Items[listBox1.SelectedIndex];
                        //paste item in selectedCell
                    }//end if selectedCell in 3rd colum (c==2)
                }//*******************end if number of input is 2*********************//
                if (nbin == 1)    /*** if number of input is 1 ***/
                {
                    if (c == 0)//if selectedCell in 1st column
                    {
                        if (dataGridView1.Rows[r].Cells[c].Value == null)//if selectedCell in "Liste" table is empty
                        {
                                ruleexists = 0;
                                rowcnt = dataGridView1.RowCount;
                                for (int p = 0; p < rowcnt; p++)
                                {
                                    // check whether the new desired rule to be added is already exist by comparing with each rule//
                                    if (dataGridView1.Rows[p].Cells[0].Value != null)
                                        if (dataGridView1.Rows[p].Cells[0].Value.ToString() == listBox1.Items[listBox1.SelectedIndex].ToString())
                                        {
                                            // show message "the rule is already exist" //
                                            MessageBox.Show(warnungdopplteRegel1);
                                            ruleexists++;       
                                        }
                                }//end for
                                if (ruleexists == 0) dataGridView1.Rows[r].Cells[c].Value = listBox1.Items[listBox1.SelectedIndex];
                                //if ruleexists = 0 paste item in selectedCell
                        }//end if selectedCell in "Liste" table is empty
                        else   //dataGridView1.Rows[r].Cells[c].Value != null
                        {
                            ruleexists = 0;
                            rowcnt = dataGridView1.RowCount;
                            for (int p = 0; p < rowcnt; p++)
                            {
                                // check whether the new desired rule to be added is already exist by comparing with each rule//
                                if (dataGridView1.Rows[p].Cells[0].Value != null)
                                    if (dataGridView1.Rows[p].Cells[0].Value.ToString() == listBox1.Items[listBox1.SelectedIndex].ToString())
                                    {
                                        // show message "the rule is already exist" //
                                        MessageBox.Show(warnungdopplteRegel1);
                                        ruleexists++;
                                    }
                            }//end for
                            if (ruleexists == 0) dataGridView1.Rows[r].Cells[c].Value = listBox1.Items[listBox1.SelectedIndex];
                            //if ruleexists = 0 paste item in selectedCell
                        }//end else
                    }//end if selectedCell in 1st column
                    if (c == 1)//if selectedCell in 2nd column
                    {
                        dataGridView1.Rows[r].Cells[c].Value = listBox1.Items[listBox1.SelectedIndex];
                        //paste item in selectedCell
                    }
                } //end if number of input is 1
            }//******************end if selected Tab is "Liste"*********************//

            if (tabControl1.SelectedIndex == 1)//if selected Tab is "Matrix"
            {
                if (dataGridView2.Rows[rr].Cells[cc].Value == null)//if selectedCell is empty in matrix table
                {
                    dataGridView2.Rows[rr].Cells[cc].Value = listBox1.Items[listBox1.SelectedIndex];
                    //paste item in selectedCell in Matrix table
                    //paste this new rule to new line in "Liste" table 
                    dataGridView1.Rows[nbrule].Cells[0].Value = dataGridView2.Rows[rr].HeaderCell.Value;
                    dataGridView1.Rows[nbrule].Cells[1].Value = dataGridView2.Columns[cc].HeaderCell.Value;
                    dataGridView1.Rows[nbrule].Cells[2].Value = dataGridView2.Rows[rr].Cells[cc].Value;
                    dataGridView1.Rows[nbrule].Cells[3].Value = 100;

                    nbrule = 0;//set number of rules = 0
                    for (int n = 0; n < dataGridView1.RowCount; n++)
                    {
                        //check each rule in Liste table
                        if (dataGridView1.Rows[n].Cells[0].Value != null)
                        {
                            if (dataGridView1.Rows[n].Cells[1].Value != null)
                            {
                                if (dataGridView1.Rows[n].Cells[2].Value != null)
                                {
                                    if (dataGridView1.Rows[n].Cells[3].Value != null)
                                    {
                                        int gew = (int)Convert.ToDouble(dataGridView1.Rows[n].Cells[3].Value.ToString());
                                        rule[3, n] = gew;//update the value in array
                                        //value from 0 up to 100 is allowed for weight(gew)
                                        if (rule[3, n] > 100)
                                        {
                                            rule[3, n] = 100;//make sure weight is not greater than 100
                                        }
                                        if (rule[3, n] < 0)
                                        {
                                            rule[3, n] = 0;//make sure weight is not negative value
                                        }

                                        nbrule = n + 1;//current number of rules
                                        label3.Text = nbrule.ToString() + " Rules";//display number of rules
                                        for (int f = 0; f < nbsets[0]; f++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[0].Value.ToString() == names[0, f])
                                                rule[0, n] = f + 1;//update the value in array
                                        }
                                        for (int g = 0; g < nbsets[1]; g++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[1].Value.ToString() == names2[0, g])
                                                rule[1, n] = g + 1;//update the value in array
                                        }
                                        for (int h = 0; h < nbsets[2]; h++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[2].Value.ToString() == Outnames[0, h])
                                                rule[2, n] = h + 1;//update the value in array
                                        }
                                    }//end if
                                }
                            }
                        }
                    }//end for                   
                    buttonSave_Click(sender, null); //save
                    buttonDrawArray_Click(sender, null);//draw fuzzy from arrays

                    List<string> listItems = new List<string>();
                    listItems = null;
                    listBox1.DataSource = listItems;//clear listbox1
                    List<string> listItems2 = new List<string>();
                    for (int a = 0; a < richTextBox3.Lines.Length; a++)
                        listItems2.Add(richTextBox3.Lines[a]);
                        listBox1.DataSource = listItems2;//display output's names list in listbox1
                }//end if selectedCell is empty in matrix table
            } //end if selected Tab is "Matrix"       
        }

        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (nbin == 1)//if number of input is 1
            {
                if (tabControl1.SelectedIndex == 1)//show "Liste" table although "Matrix" is clicked
                {
                    tabControl1.SelectedTab = Liste;
                }
            }//end if
            if (nbin == 2)//if number of input is 2
            {
                if (textBox5.Text.Length == 0)
                {
                    buttonDraw_Click(sender, null);//draw fuzzysets
                }
                else//if textbox5.text != 0
                {
                    buttonDrawArray_Click(sender, null);//draw fuzzysets reffering to datasets in arrays
                }//end else
                buttonSave_Click(sender, null); //save
                List<string> listItems = new List<string>();
                listItems = null;
                listBox1.DataSource = listItems;//delete items in listbox1
                dataGridView1.ClearSelection();//clear selection in "Liste"
                dataGridView2.ClearSelection();//clear selection in "Matrix"
            }//end if
        }
        
        public void buttonDrawArray_Click(object sender, EventArgs e) //draw from arrays
        {
            // draw using data from arrays //
            textBox1.Text = textBox5.Text;
            buttonDraw_Click(sender, null);
        }

        private void FormFuzEditor_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
            button_clear_Click(sender, null);//clear everything in FormFuzEditor
            Hide();
        }
        
        private void buttonDeleteRule_Click(object sender, EventArgs e)
        {
            // delete only the selected rule when entire row is selected //
            //************delete rule that has been selected in "Liste" table******************//
            if (tabControl1.SelectedIndex == 0)//if selectedTab is "Liste" table
            { 
                if(nbin==2)//if number of input is 2
                {
                    label9.Text = dataGridView1.SelectedRows.Count.ToString();
                    if (dataGridView1.SelectedRows.Count == 1)//if entire row is selected
                    {
                    // delete selected rule in both tables
                    dataGridView2.Rows[rule[0, r] - 1].Cells[rule[1, r] - 1].Value = null;
                    dataGridView1.Rows[r].Cells[0].Value = null;
                    dataGridView1.Rows[r].Cells[1].Value = null;
                    dataGridView1.Rows[r].Cells[2].Value = null;
                    dataGridView1.Rows[r].Cells[3].Value = null;
                    }
                    // recount the total of rules and update values in arrays by checking //
                    // each line of rest rules in "Liste"                                 //
                    int i = 0;//set initial value of new nbrule = 0
                    for (int n = 0; n < nbrule; n++)
                    {
                    if (dataGridView1.Rows[n].Cells[0].Value != null)
                    {
                        if (dataGridView1.Rows[n].Cells[1].Value != null)
                        {
                            if (dataGridView1.Rows[n].Cells[2].Value != null)
                            {
                                if (dataGridView1.Rows[n].Cells[3].Value != null)
                                {
                                    i = i + 1;
                                    int gew = (int)Convert.ToDouble(dataGridView1.Rows[n].Cells[3].Value.ToString());
                                    rule[3, i - 1] = gew;
                                    
                                    dataGridView1.Rows[i - 1].Cells[3].Value = dataGridView1.Rows[n].Cells[3].Value;

                                    for (int f = 0; f < nbsets[0]; f++)
                                    {
                                        if (dataGridView1.Rows[n].Cells[0].Value.ToString() == names[0, f])
                                            rule[0, i - 1] = f + 1;
                                            dataGridView1.Rows[i - 1].Cells[0].Value = names[0, f];
                                    }
                                    for (int g = 0; g < nbsets[1]; g++)
                                    {
                                        if (dataGridView1.Rows[n].Cells[1].Value.ToString() == names2[0, g])
                                            rule[1, i - 1] = g + 1;
                                            dataGridView1.Rows[i - 1].Cells[1].Value = names2[0, g];
                                    }
                                    for (int h = 0; h < nbsets[2]; h++)
                                    {
                                        if (dataGridView1.Rows[n].Cells[2].Value.ToString() == Outnames[0, h])
                                            rule[2, i - 1] = h + 1;
                                            dataGridView1.Rows[i - 1].Cells[2].Value = Outnames[0, h];
                                    }
                                }
                            }
                        }
                    }
                    }
                    if (nbrule > 0)
                    {
                        //delete last line of old rulebase List
                        dataGridView1.Rows[nbrule - 1].Cells[0].Value = null;
                        dataGridView1.Rows[nbrule - 1].Cells[1].Value = null;
                        dataGridView1.Rows[nbrule - 1].Cells[2].Value = null;
                        dataGridView1.Rows[nbrule - 1].Cells[3].Value = null;
                        nbrule = i;//new nbrule
                        label3.Text = nbrule.ToString() + " Rules";//paste nbrule
                        buttonSave_Click(sender, null);//save 
                        buttonDrawArray_Click(sender, null);//draw from arrays
                    }
                    else
                    {
                        if (nbrule == 0)//if no rules left after deleting
                        {
                            nbrule = 0;//define current nbrule = 0
                            label3.Text = nbrule.ToString() + " Rules";//paste nbrule
                            buttonSave_Click(sender, null);//save
                            buttonDrawArray_Click(sender, null);//draw from arrays
                        }
                    }
            }//end nbin=2
                if (nbin == 1)//if number of inputs is 1
                {
                    label9.Text = dataGridView1.SelectedRows.Count.ToString();
                    if (dataGridView1.SelectedRows.Count == 1)//and one rule is selected
                    {
                        //delete selected rule in "Liste" table
                        dataGridView1.Rows[r].Cells[0].Value = null;
                        dataGridView1.Rows[r].Cells[1].Value = null;
                        dataGridView1.Rows[r].Cells[2].Value = null;
                    }
                    // recount the total of rules and update values in arrays by checking //
                    // each line of rest rules in "Liste"                                 //
                    int i = 0;//set initial value of new nbrule = 0
                    for (int n = 0; n < nbrule; n++)
                    {
                        if (dataGridView1.Rows[n].Cells[0].Value != null)
                        {
                            if (dataGridView1.Rows[n].Cells[1].Value != null)
                            {
                                if (dataGridView1.Rows[n].Cells[2].Value != null)
                                {
                                        i = i + 1;
                                        int gew = (int)Convert.ToDouble(dataGridView1.Rows[n].Cells[2].Value.ToString());
                                        rule[3, i - 1] = gew;

                                        dataGridView1.Rows[i - 1].Cells[3].Value = dataGridView1.Rows[n].Cells[2].Value;

                                        for (int f = 0; f < nbsets[0]; f++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[0].Value.ToString() == names[0, f])
                                                rule[0, i - 1] = f + 1;
                                                dataGridView1.Rows[i - 1].Cells[0].Value = names[0, f];
                                        }
                                        for (int h = 0; h < nbsets[2]; h++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[2].Value.ToString() == Outnames[0, h])
                                                rule[2, i - 1] = h + 1;
                                                dataGridView1.Rows[i - 1].Cells[2].Value = Outnames[0, h];
                                        }
                                }
                            }
                        }
                    }
                    if (nbrule > 0)
                    {
                        //delete last line of old rulebase List
                        dataGridView1.Rows[nbrule - 1].Cells[0].Value = null;
                        dataGridView1.Rows[nbrule - 1].Cells[1].Value = null;
                        dataGridView1.Rows[nbrule - 1].Cells[2].Value = null;
                        nbrule = i;//new nbrule
                        label3.Text = nbrule.ToString() + " Rules";//paste nbrule
                        buttonSave_Click(sender, null);
                        buttonDrawArray_Click(sender, null);
                    }
                    else
                    {
                        if (nbrule == 0)//if no rules left after deleting
                        {
                            nbrule = 0;//define current nbrule = 0
                            label3.Text = nbrule.ToString() + " Rules";//and paste nbrule
                            buttonSave_Click(sender, null);
                            buttonDrawArray_Click(sender, null);
                        }
                    }
                }
            }//end deleting rule selected in "LIste"//
            //************delete rule that has been selected in "Matrix" table******************//
            if (tabControl1.SelectedIndex == 1)//if selectedTab is "Matrix" table
            {
                label9.Text = dataGridView2.SelectedCells.Count.ToString();
                
                if (dataGridView2.SelectedCells.Count == 1)//if a rule is selected in "Matrix" table
                {
                    dataGridView2.Rows[rr].Cells[cc].Value = null;//delete that rule 
                    // then delete also the corresponding rule in "Liste" //
                    for (int kp = 0; kp < nbrule; kp++)
                    {
                        if (dataGridView1.Rows[kp].Cells[0].Value.ToString() == dataGridView2.Rows[rr].HeaderCell.Value.ToString())
                        {
                            if (dataGridView1.Rows[kp].Cells[1].Value.ToString() == dataGridView2.Columns[cc].HeaderCell.Value.ToString())
                            {
                                dataGridView1.Rows[kp].Cells[0].Value = null;
                                dataGridView1.Rows[kp].Cells[1].Value = null;
                                dataGridView1.Rows[kp].Cells[2].Value = null;
                                dataGridView1.Rows[kp].Cells[3].Value = null;
                            }
                        }
                    }
                    // recount the total of rules and update values in arrays by checking //
                    // each line of rest rules in "Liste"                                 //
                    int i = 0;//set initial value of new nbrule = 0
                    for (int n = 0; n < nbrule; n++)
                    {
                        if (dataGridView1.Rows[n].Cells[0].Value != null)
                        {
                            if (dataGridView1.Rows[n].Cells[1].Value != null)
                            {
                                if (dataGridView1.Rows[n].Cells[2].Value != null)
                                {
                                    if (dataGridView1.Rows[n].Cells[3].Value != null)
                                    {
                                        i = i + 1;
                                        int gew = (int)Convert.ToDouble(dataGridView1.Rows[n].Cells[3].Value.ToString());
                                        rule[3, i - 1] = gew;
                                        
                                        dataGridView1.Rows[i - 1].Cells[3].Value = dataGridView1.Rows[n].Cells[3].Value;

                                        for (int f = 0; f < nbsets[0]; f++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[0].Value.ToString() == names[0, f])
                                                rule[0, i - 1] = f + 1;
                                            dataGridView1.Rows[i - 1].Cells[0].Value = names[0, f];
                                        }
                                        for (int g = 0; g < nbsets[1]; g++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[1].Value.ToString() == names2[0, g])
                                                rule[1, i - 1] = g + 1;
                                            dataGridView1.Rows[i - 1].Cells[1].Value = names2[0, g];
                                        }
                                        for (int h = 0; h < nbsets[2]; h++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[2].Value.ToString() == Outnames[0, h])
                                                rule[2, i - 1] = h + 1;
                                            dataGridView1.Rows[i - 1].Cells[2].Value = Outnames[0, h];
                                        }
                                    }
                                }
                            }
                        }
                    }
                    if (nbrule > 0)
                    {
                        //delete last line of old rulebase List
                        dataGridView1.Rows[nbrule - 1].Cells[0].Value = null;
                        dataGridView1.Rows[nbrule - 1].Cells[1].Value = null;
                        dataGridView1.Rows[nbrule - 1].Cells[2].Value = null;
                        dataGridView1.Rows[nbrule - 1].Cells[3].Value = null;
                        nbrule = i;//new nbrule
                        label3.Text = nbrule.ToString() + " Rules";
                        buttonSave_Click(sender, null);//save current data
                        buttonDrawArray_Click(sender, null);//draw from arrays
                    }
                    else
                    {
                        if (nbrule == 0)//if no rules left after deleting
                        {
                            nbrule = 0;//define current nbrule = 0
                            label3.Text = nbrule.ToString() + " Rules";//and paste nbrule
                            buttonSave_Click(sender, null);
                            buttonDrawArray_Click(sender, null);
                        }
                    }
                    List<string> listItems = new List<string>();
                    listItems = null;
                    listBox1.DataSource = listItems;
                    List<string> listItems2 = new List<string>();
                    for (int a = 0; a < richTextBox3.Lines.Length; a++)
                        listItems2.Add(richTextBox3.Lines[a]);
                        listBox1.DataSource = listItems2;//listbox1 shows output's names
                }
            }//end deleting rule in "Matrix"
        }

        bool changeOK = false;
        private void dataGridView1_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            //******update values in arrays when new rule is added in Liste table******//
            if (changeOK)
            {
                changeOK = false;

                if (nbin == 2)//if number of input is 2
                {
                    nbrule = 0;//set number of rules = 0
                    for (int n = 0; n < dataGridView1.RowCount; n++)
                    {
                        //check each rule in liste table
                        if (dataGridView1.Rows[n].Cells[0].Value != null)
                        {
                            if (dataGridView1.Rows[n].Cells[1].Value != null)
                            {
                                if (dataGridView1.Rows[n].Cells[2].Value != null)
                                {
                                    if (dataGridView1.Rows[n].Cells[3].Value != null)
                                    {
                                        int gew = 100;
                                        Int32.TryParse(dataGridView1.Rows[n].Cells[3].Value.ToString(), out gew);
                                        rule[3, n] = gew;//update the value in array

                                        if (rule[3, n] > 100)
                                        {
                                            rule[3, n] = 100;
                                        }
                                        if (rule[3, n] < 0)
                                        {
                                            rule[3, n] = 0;
                                        }

                                        nbrule = n + 1;//current number of rules
                                        label3.Text = nbrule.ToString() + " Rules";//display current number of rules
                                        for (int f = 0; f < nbsets[0] ; f++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[0].Value.ToString() == names[0, f])
                                                rule[0, n] = f + 1;//update the value in array
                                        }
                                        for (int g = 0; g < nbsets[1]; g++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[1].Value.ToString() == names2[0, g])
                                                rule[1, n] = g + 1;//update the value in array
                                        }
                                        for (int h = 0; h < nbsets[2]; h++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[2].Value.ToString() == Outnames[0, h])
                                                rule[2, n] = h + 1;//update the value in array
                                        }
                                    }
                                }
                            }
                        }
                    }
                    // check if new rule is added //
                    if (dataGridView1.Rows[r].Cells[0].Value != null)
                    {
                        if (dataGridView1.Rows[r].Cells[1].Value != null)
                        {
                            if (dataGridView1.Rows[r].Cells[2].Value != null)
                            {
                                if (dataGridView1.Rows[r].Cells[3].Value != null)
                                {
                                    //save and draw using values in arrays if new rule is added// 
                                    buttonSave_Click(sender, null);
                                    buttonDrawArray_Click(sender, null);
                                }
                            }
                        }
                    }//end check new rule
                }//end if number of input is 2
                if (nbin == 1)//if number of input is 1
                {
                    //the program method here is the same as above//
                    nbrule = 0;
                    for (int n = 0; n < dataGridView1.RowCount; n++)
                    {
                        if (dataGridView1.Rows[n].Cells[0].Value != null)
                        {
                            if (dataGridView1.Rows[n].Cells[1].Value != null)
                            {
                                if (dataGridView1.Rows[n].Cells[2].Value != null)
                                {
                                        int gew = 100;
                                        Int32.TryParse(dataGridView1.Rows[n].Cells[2].Value.ToString(), out gew);
                                        rule[3, n] = gew;

                                        if (rule[3, n] > 100)
                                        {
                                            rule[3, n] = 100;
                                        }
                                        if (rule[3, n] < 0)
                                        {
                                            rule[3, n] = 0;
                                        }

                                        nbrule = n + 1;
                                        label3.Text = nbrule.ToString() + " Rules";
                                        for (int f = 0; f < nbsets[0]; f++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[0].Value.ToString() == names[0, f])
                                                rule[0, n] = f + 1;
                                        }
                                        for (int h = 0; h < nbsets[2]; h++)
                                        {
                                            if (dataGridView1.Rows[n].Cells[1].Value.ToString() == Outnames[0, h])
                                                rule[2, n] = h + 1;
                                        }
                                }
                            }
                        }
                    }
                    if (dataGridView1.Rows[r].Cells[0].Value != null)
                    {
                        if (dataGridView1.Rows[r].Cells[1].Value != null)
                        {
                            if (dataGridView1.Rows[r].Cells[2].Value != null)
                            {
                                    buttonSave_Click(sender, null);
                                    buttonDrawArray_Click(sender, null);  
                            }
                        }
                    }
                }//end if number of input is 1
            }                   
        }  
      
        private void rBOutFuzZero_CheckedChanged(object sender, EventArgs e)
        {
            // if radiobutton 'zero' is checked, then outproc="zero" //
            if (rBOutFuzZero.Checked)
                outproc = "zero";
            else
                // if radiobutton 'hold' is checked, then outproc="hold" //
                outproc = "hold";
        } 
       
        private void buttonDelAllRules_Click(object sender, EventArgs e)
        {
            // delete all rules in tables (Liste and Matrix) //
            dataGridView1.Rows.Clear();
            dataGridView2.Rows.Clear();

            nbrule = 0;
            label3.Text = nbrule.ToString() + " Rules";
            buttonSave_Click(sender, null);
            buttonDrawArray_Click(sender, null);

            dataGridView2.ClearSelection();
            List<string> listItems = new List<string>();
            listItems = null;
            listBox1.DataSource = listItems;
        }

        private void buttonSaveAs_Click(object sender, EventArgs e)
        {
            // save current data in new fuz. file //
            buttonSave_Click(sender, null); //button Save
            if (saveFileDialog2.ShowDialog() == DialogResult.OK)
            {
                FileStream fs2 = new FileStream(saveFileDialog2.FileName, FileMode.Create);
                StreamWriter myStream3 = new
                StreamWriter(fs2);
                myStream3.WriteLine(textBox5.Text);
                myStream3.Close();
            }
        }

        private void button_clear_Click(object sender, EventArgs e)
        {
            // clear all textboxes //
            textBox1.Clear();
            textBox5.Clear();
            // clear all labels //
            label1.Text = null;
            label2.Text = null;
            label3.Text = null;
            label4.Text = null;
            label5.Text = null;
            label6.Text = null;
            label7.Text = null;
            label7.Text = null;
            label8.Text = null;
            label9.Text = null;
            // clear all lists of Fuzzy sets //
            richTextBox1.Text = "";
            richTextBox2.Text = "";
            richTextBox3.Text = "";
            // clear all Fuzzy sets //
            SolidBrush clear = new SolidBrush(Color.White);
            gPB1.FillRectangle(clear, 0, 0, panel1.Width, panel1.Height);
            gPB2.FillRectangle(clear, 0, 0, panel2.Width, panel2.Height);
            gPB3.FillRectangle(clear, 0, 0, panel3.Width, panel3.Height);
            gPB4.FillRectangle(clear, 0, 0, panel4.Width, panel4.Height);
            gPB5.FillRectangle(clear, 0, 0, panel5.Width, panel5.Height);
            gPB6.FillRectangle(clear, 0, 0, panel6.Width, panel6.Height);
            // clear rule base (Liste) //
            dataGridView1.Rows.Clear();
            dataGridView1.Columns.Clear();
            // clear rule base (Matrix) //
            dataGridView2.Rows.Clear();
            dataGridView2.Columns.Clear();
            // clear runtime //
            labFuzOut.Text ="Out= ...";
            textBoxIn1.Text = "0";
            textBoxIn2.Text = "0";
            rBOutFuzZero.Checked = true;
            rBOutFuzHold.Checked = false;
            // clear listbox //
            List<string> listItems = new List<string>();
            listItems = null;
            listBox1.DataSource = listItems;
        }
     }
}

